/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.naming;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NameParser;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;
import javax.naming.spi.ObjectFactory;
import javax.naming.spi.StateFactory;

/**
 * This is the default namespace implementation.
 *
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.2 $
 */
public class DefaultNamespace
    extends AbstractNamespace
{
    protected NameParser             m_nameParser;

    public DefaultNamespace( final NameParser nameParser )
    {
        this( nameParser,
              new ObjectFactory[ 0 ],
              new StateFactory[ 0 ] );
    }

    public DefaultNamespace( final NameParser nameParser,
                             final ObjectFactory[] objectFactorySet,
                             final StateFactory[] stateFactorySet )
    {
        m_nameParser = nameParser;
        m_objectFactorySet = objectFactorySet;
        m_stateFactorySet = stateFactorySet;
    }

    public synchronized void addStateFactory( final StateFactory stateFactory )
    {
        //create new array of factory objects
        final StateFactory[] stateFactorySet =
            new StateFactory[ m_stateFactorySet.length + 1 ];

        //copy old factory objects to new array
        System.arraycopy( m_stateFactorySet, 0, stateFactorySet, 0, m_stateFactorySet.length );

        //add in new factory at end
        stateFactorySet[ m_stateFactorySet.length ] = stateFactory;

        //update factory set
        m_stateFactorySet = stateFactorySet;
    }

    public synchronized void addObjectFactory( final ObjectFactory objectFactory )
    {
        //create new array of factory objects
        final ObjectFactory[] objectFactorySet =
            new ObjectFactory[ m_objectFactorySet.length + 1 ];

        //copy old factory objects to new array
        System.arraycopy( m_objectFactorySet, 0, objectFactorySet, 0, m_objectFactorySet.length );

        //add in new factory at end
        objectFactorySet[ m_objectFactorySet.length ] = objectFactory;

        //update factory set
        m_objectFactorySet = objectFactorySet;
    }

    public NameParser getNameParser()
    {
        return m_nameParser;
    }
}
