/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: HttpJmsAdminConnection.java,v 1.10 2003/08/18 10:16:12 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date        jimm    Created
 */
package org.exolab.jms.administration.http;


import javax.jms.JMSException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.exolab.core.http.HttpClient;
import org.exolab.jms.administration.mipc.IpcJmsAdminConnection;


/**
 * This class is repsonsible for opening an http connection to the server
 * and passing and packing all requests for delivery.
 *
 * @version     $Revision: 1.10 $ $Date: 2003/08/18 10:16:12 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         org.exolab.core.http.HttpClient
 */
public class HttpJmsAdminConnection extends IpcJmsAdminConnection {

    /**
     * The admin servlet path
     */
    private String _path;

    /**
     * The logger
     */
    private static final Log _log = LogFactory.getLog(
        HttpJmsAdminConnection.class);


    /**
     * Create the connection to the server
     *
     * @param host the webserver host
     * @param port the webserver port
     * @param path the admin servlet path. May be null
     * @param username the admin user name
     * @param password the admin password
     * @throws JMSException if a connection cannot be established
     */
    public HttpJmsAdminConnection(String host, int port, String path,
                                  String username, String password)
        throws JMSException {
        super(host, port);

        _path = path;
        if (_path == null) {
            _path = "/openjms/OpenJMSServer";
        } else if (!_path.startsWith("/")) {
            _path = "/" + _path;
        }
        connect(username, password);
    }

    // implementation of JmsAdminServerIfc.close
    public void close() {
        if (getConnection() != null) {
            try {
                getConnection().close();
            } catch (Exception exception) {
                _log.debug("Exception during close", exception);
            }
            setConnection(null);
        }
    }

    /**
     * Create HTTP client connection.
     *
     * @param host The client to connect to
     * @param port The port used to connect to the client above
     * @throws JMSException If the connection fails.
     */
    protected void openConnection() throws JMSException {
        try {
            String url = getURL("http", getHost(), getPort());
            setConnection(new HttpClient(url, "HttpJmsAdminConnection"));
        } catch (Exception exception) {
            throw new JMSException(exception.getMessage());
        }
    }

    /**
     * Returns the admin servlet path
     *
     * @return the admin servlet path
     */
    protected String getPath() {
        return _path;
    }

    /**
     * Returns the admin servlet URL
     *
     * @param scheme the connection protocol
     * @param host the webserver host
     * @param port the webserver port
     * @return the admin servlet URL
     */
    protected String getURL(String scheme, String host, int port) {
        return scheme + "://" + host + ":" + port + getPath();
    }

} //-- HttpJmsAdminConnection
