/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SslIpcJmsAdminConnection.java,v 1.10 2003/08/07 13:32:48 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date        jimm    Created
 */
package org.exolab.jms.administration.mipc;

import java.io.IOException;
import java.lang.reflect.Constructor;

import javax.jms.JMSException;

import org.exolab.core.mipc.MultiplexConnectionIfc;


/**
 * This class is repsonsible for opening an SSL TCP connection to the
 * server and passing and packing all requests for delivery.
 *
 * @version     $Revision: 1.10 $ $Date: 2003/08/07 13:32:48 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 */
public class SslIpcJmsAdminConnection extends IpcJmsAdminConnection {

    /**
     * Construct a new <code>SslIpcJmsAdminConnection</code>,
     * establishing a connection to the server
     *
     * @param host the server host
     * @param port the server port
     * @param username the admin user name
     * @param password the admin password
     * @throws JMSEXception if a connection cannot be established
     */
    public SslIpcJmsAdminConnection(String host, int port, String username,
                                    String password) throws JMSException {
        super(host, port, username, password);
    }

    /**
     * Create an SSL connection.
     *
     * @param host the address of the server
     * @param port the port number to use
     * @return a new connection
     * @throws IOException if the server fails to initialise the ip service
     */
    protected MultiplexConnectionIfc createClientConnection(
        String host, int port) throws IOException {
        MultiplexConnectionIfc con = null;

        try {
            Class serverClass = Class.forName(
                "org.exolab.core.mipc.MultiplexSSLConnection");

            Class[] types = new Class[]{String.class, int.class};
            Object[] args = new Object[]{host, new Integer(port)};
            Constructor constructor = serverClass.getConstructor(types);
            con = (MultiplexConnectionIfc) constructor.newInstance(args);
        } catch (Exception exception) {
            throw new IOException(exception.getMessage());
        }
        return con;
    }

} //-- SslIpcJmsAdminConnection
