/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsClientStubFactory.java,v 1.3 2003/08/07 13:32:49 tanderson Exp $
 *
 * Date         Author  Changes
 * 3/21/2000    jima    Created
 */
package org.exolab.jms.client;

import javax.jms.JMSException;


/**
 * This is a helper clas that is responsible for cresting the various types of
 * stubs. The client requires three stubs one to talk to the remote jms server
 * (i.e connection factory) one to talk to the connection and another to talk
 * to the session.
 * <p>
 * This classrelies on configuration information to determine which ORB to
 * use.
 *
 * @version     $Revision: 1.3 $ $Date: 2003/08/07 13:32:49 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
final public class JmsClientStubFactory {

    /**
     * This class method creates and returns a stub to the remote server
     * entity. This entity must support the JmsServerStubIfc interface
     * <p>
     * If there is problem creating the stub then throw the JMSException
     * exception
     *
     * @param       id              client identity
     * @exception   JMSException
     */
    public static JmsServerStubIfc createJmsServerStub()
        throws JMSException {
        // dynamincally instantiate an instance of this class and then set the
        // client id. Finally, establish the connection
        return null;
    }

    /**
     * This class method creates and returns a stub to the remote connection
     * entity. This entity must support the JmsConnectionStubIfc interface
     * It uses the client identity to initialise the connection.
     * <p>
     * If there is problem creating the stub then throw the JMSException
     * exception
     *
     * @param       id              client identity
     * @exception   JMSException
     */
    public static JmsConnectionStubIfc createJmsConnectionStub(String id)
        throws JMSException {
        // will hard code the class name, but once we have sorted out
        // the type of configuration file we will need to retrieve it from
        // a property
        // dynamincally instantiate an instance of this class and then set the
        // client id. Finally, establish the connection
        return null;
    }

    /**
     * This class method creates and returns a stub to the rmeote session
     * entity. The created entity must support the JmsSessionStubIfc interface.
     * This level of indirection will assist us in supported different ORB
     * environments
     * <p>
     * If there is a problem creating the stub then throw the JMSException
     * exception
     *
     * @param       id              client identity
     * @exception   JMSException
     */
    public static JmsSessionStubIfc createJmsSessionStub(String id)
        throws JMSException {
        // dynamincally instantiate an instance of this class and then set the
        // client id. Finally, establish the connection
        return null;
    }


    /**
     * This is the name of the server stub. This will be passed in as a
     * configuraton parameter in the future
     */
    final private static String SERVER_CLASS =
        "org.exolab.jms.server.rmi.RemoteJmsServerIfc";

    /**
     * This is the name of the connection stub that will be used to gain access
     * to the remote connection. This will be passed as a configuration
     * parameter in future.
     */
    final private static String CONNECTION_CLASS =
        "org.exolab.jms.server.rmi.RemoteJmsServerConnectionIfc";

    /**
     * This is the name of the session stub that will be used to gain access to the
     * remote session. This will be passed in as a configuration paramter in
     * future
     */
    final private static String SESSION_CLASS =
        "org.exolab.jms.server.rmi.RemoteJmsServerSessionIfc";
}
