/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsConnectionFactoryBuilder.java,v 1.4 2003/08/07 13:32:49 tanderson Exp $
 *
 * Date         Author  Changes
 * 06/14/2001   jima    Created
 */
package org.exolab.jms.client;


import java.util.Enumeration;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import javax.naming.spi.ObjectFactory;


/**
 * This factory object implements the ObjectFactory interface and is used
 * to resolve a reference to a connection factory. This is part
 * of our JNDI support.
 * <p>
 * All {@link JmsConnectionFactory} objects support the java.naming.Referenceable
 * interface and make explicit reference to this factory
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:32:49 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 *
 * @see         org.exolab.jms.client.JmsTopicConnectionFactory
 * @see         org.exolab.jms.client.JmsQueueConnectionFactory
 **/
public class JmsConnectionFactoryBuilder
    implements ObjectFactory {


    // implementation of ObjectFactory.getObjectInstance
    public Object getObjectInstance(Object object, Name name, Context context,
                                    Hashtable env) throws Exception {
        Object result = null;

        if (object instanceof Reference) {
            Reference ref = (Reference) object;
            String className = ref.getClassName();

            if (className.equals(JmsQueueConnectionFactory.class.getName()) ||
                className.equals(JmsTopicConnectionFactory.class.getName())) {

                StringRefAddr serverClass =
                    (StringRefAddr) ref.get("serverClass");

                // get a list of string properties and store them in a
                // hashtable
                Hashtable props = new Hashtable();
                Enumeration iter = ref.getAll();
                while (iter.hasMoreElements()) {
                    StringRefAddr addr = (StringRefAddr) iter.nextElement();
                    props.put(addr.getType(), addr.getContent());
                }

                // create the factory
                if (className.equals(
                    JmsQueueConnectionFactory.class.getName())) {
                    result = new JmsQueueConnectionFactory(
                        (String) serverClass.getContent(), props);
                } else {
                    result = new JmsTopicConnectionFactory(
                        (String) serverClass.getContent(), props);
                }
            }
        }
        return result;
    }
}

