/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsConnectionStubIfc.java,v 1.8 2003/08/17 01:32:21 tanderson Exp $
 *
 * Date         Author  Changes
 * 04/12/2000    jima    Created
 */
package org.exolab.jms.client;

import javax.jms.JMSException;


/**
 * This is the interface that connection stubs must implement in order to
 * support remote invocations. This level of indirection will enable us to
 * support different ORB environments. The only restriction is that the
 * stubs must support this interface and that it must have a default
 * constructor.
 *
 * @version     $Revision: 1.8 $ $Date: 2003/08/17 01:32:21 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 */
public interface JmsConnectionStubIfc {

    /**
     * Create a session.
     *
     * @param ackMode the session ack mode
     * @param  transacted <code>true</code> if it is a transacted session
     * @throws JMSException for any JMS error
     */
    JmsSessionStubIfc createSession(int ackMode, boolean transacted)
        throws JMSException;

    /**
     * Close the connection and deallocate any resources.
     *
     * @throws JMSException for any JMS error
     */
    void close() throws JMSException;

    /**
     * Release all local resources used by  this object.
     *
     * @throws JMSException for any JMS error
     */
    void destroy() throws JMSException;

    /**
     * Retrieve the identity of this conection.
     *
     * @return the identity of this conection
     * @throws JMSException for any JMS error
     */
    String getConnectionId() throws JMSException;
}
