/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2004 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsQueueConnectionConsumer.java,v 1.5 2004/01/11 02:33:44 tanderson Exp $
 */
package org.exolab.jms.client;

import javax.jms.ConnectionConsumer;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.ServerSessionPool;
import javax.jms.Session;


/**
 * Client implementation of the <code>javax.jms.ConnectionConsumer</code>
 * interface, for queues.
 *
 * @version     $Revision: 1.5 $ $Date: 2004/01/11 02:33:44 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @author      <a href="mailto:tma@netspace.net.au">Tim Anderson</a>
 */
class JmsQueueConnectionConsumer extends JmsConnectionConsumer {

    /**
     * The session to receive messages via.
     */
    private QueueSession _session;

    /**
     * The receiver of messages
     */
    private QueueReceiver _receiver;


    /**
     * Construct a new <code>JmsQueueConnectionConsumer</code>
     *
     * @param connection the connection which created this
     * @param queue the queue to access
     * @param pool the server session pool
     * @param selector the message selector. May be <code>null</code>
     * @param maxMessages the maximum number of messages that can be assigned 
     * to a server session at one time
     * @throws JMSException if the consumer cannot be constructed
     */
    public JmsQueueConnectionConsumer(QueueConnection connection, Queue queue,
                                      ServerSessionPool pool, String selector, 
                                      int maxMessages)
        throws JMSException {
        super(pool, maxMessages);

        _session = connection.createQueueSession(
            false, Session.CLIENT_ACKNOWLEDGE);
        _receiver = _session.createReceiver(queue, selector);
        _receiver.setMessageListener(this);
    }

    /**
     * Close the connection consumer, freeing any allocated resources
     *
     * @throws JMSException if the consumer cannot be closed
     */
    public void close() throws JMSException {
        super.close();
        try {
            _receiver.close();
            _session.close();
        } finally {
            _receiver = null;
            _session = null;
        }
    }

}
