/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2004 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsQueueReceiver.java,v 1.11 2004/01/20 14:14:21 tanderson Exp $
 */
package org.exolab.jms.client;

import javax.jms.InvalidDestinationException;
import javax.jms.QueueReceiver;
import javax.jms.Queue;
import javax.jms.JMSException;


/**
 * Client implementation of the <code>javax.jms.QueueReceiver</code> interface
 *
 * @version     $Revision: 1.11 $ $Date: 2004/01/20 14:14:21 $
 * @author      <a href="mailto:jima@comware.com.au">Jim Alateras</a>
 * @author      <a href="mailto:tma@netspace.net.au">Tim Anderson</a>
 */
class JmsQueueReceiver
    extends JmsMessageConsumer
    implements QueueReceiver {

    /**
     * The queue to receive messages from
     */
    private JmsQueue _queue = null;


    /**
     * Construct a new <code>JmsQueueReceiver</code>
     *
     * @param session the session responsible for the consumer
     * @param clientId the session allocated consumer identifier
     * @param queue the queue to access
     * @param selector the message selector. May be <code>null</code>
     * @throws InvalidDestinationException if <code>queue</code> is null
     */
    public JmsQueueReceiver(JmsSession session, long clientId, JmsQueue queue,
                            String selector)
        throws InvalidDestinationException {
        super(session, clientId, selector);
        if (queue == null) {
            throw new InvalidDestinationException("Argument 'queue' is null");
        }
        _queue = queue;
    }

    /**
     * Returns the queue associated with this receiver
     *
     * @return the queue associated with this receiver
     */
    public Queue getQueue() {
        return _queue;
    }

    /**
     * Close the receiver.
     * This call blocks until a receive or message listener in progress has
     * completed. A blocked consumer receive call returns <code>null</code>
     * when this consumer is closed.
     *
     * @throws JMSException if this consumer can't be closed
     */
    public synchronized void close() throws JMSException {
        if (!isClosed()) {
            JmsQueueSession session = (JmsQueueSession) getSession();
            session.removeReceiver(this);
            super.close();
        }
    }

    /**
     * Release all resources used by this receiver
     *
     * @throws JMSException if this receiver can't be destroyed
     */
    public synchronized void destroy() throws JMSException {
        _queue = null;
        super.destroy();
    }
}
