/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsTopicConnectionFactory.java,v 1.15 2003/08/07 13:32:51 tanderson Exp $
 *
 * Date         Author  Changes
 * 3/21/2000    jima    Created
 */
package org.exolab.jms.client;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Hashtable;

import javax.jms.JMSException;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.naming.Referenceable;

import org.exolab.jms.util.UUID;


/**
 * This class is responsible for managing all TopicConnection instances. It
 * maintains no sta
 *

 * @version     $Revision: 1.15 $ $Date: 2003/08/07 13:32:51 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         javax.jms.TopicConnection
 **/
public class JmsTopicConnectionFactory
    extends JmsConnectionFactory
    implements TopicConnectionFactory, Externalizable, Referenceable {

    /**
     * Used for serialization
     */
    static final long serialVersionUID = 1;


    /**
     * Need a default constructor so that it can be support externalization
     */
    public JmsTopicConnectionFactory() {
    }

    /**
     * Instantiate an instance of the topic connection factory with the name
     * of the class that will facilitate communication with the JmsServer
     *
     * @param       name            name of the class
     * @param       env             environment variables
     */
    public JmsTopicConnectionFactory(String name, Hashtable env) {
        super(name, env);
    }

    // implementation of TopicConnectionFactory.createTopicConnection()
    public TopicConnection createTopicConnection()
        throws JMSException {
        return this.createTopicConnection(null, null);
    }

    // implementation of TopicConnectionFactory.createTopicConnection(String...
    public TopicConnection createTopicConnection(String userName,
                                                 String password) throws JMSException {

        // generate an id and then create a connection with the specified id.
        // Once the connection has been created register it with the factory.
        String id = UUID.next();
        JmsTopicConnection connection = new JmsTopicConnection(
            this, id, userName, password);
        addConnection(connection);
        return connection;
    }

    // implementation of Externalizable.writeExternal
    public void writeExternal(ObjectOutput stream)
        throws IOException {
        stream.writeLong(serialVersionUID);
        super.writeExternal(stream);
    }

    // implementation of Externalizable.readExternal
    public void readExternal(ObjectInput stream)
        throws IOException, ClassNotFoundException {
        long version = stream.readLong();
        if (version == serialVersionUID) {
            super.readExternal(stream);
        } else {
            throw new IOException("JmsTopicConnectionFactory with version " +
                version + " is not supported.");
        }
    }
}
