/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JmsXATopicConnectionFactory.java,v 1.5 2003/08/07 13:32:51 tanderson Exp $
 *
 * Date         Author  Changes
 * 10/09/2001   jima    Created
 */
package org.exolab.jms.client;

// java core libraries

import java.io.Externalizable;
import java.util.Hashtable;

import javax.jms.JMSException;
import javax.jms.TopicConnectionFactory;
import javax.jms.XAConnectionFactory;
import javax.jms.XATopicConnection;
import javax.jms.XATopicConnectionFactory;
import javax.naming.Referenceable;

import org.exolab.jms.util.UUID;


/**
 * An XA topic connection factory class that extends the bahaviour of
 * {@link JmsTopicConnectionFactory}.
 *
 * @version     $Revision: 1.5 $ $Date: 2003/08/07 13:32:51 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 * @see         javax.jms.XAConnectionFactory
 * @see         javax.jms.TopicConnectionFactory
 **/
public class JmsXATopicConnectionFactory
    extends JmsTopicConnectionFactory
    implements XATopicConnectionFactory, Externalizable, Referenceable {

    /**
     * This is used to mark the serializable version of this class
     */
    static final long serialVersionUID = 1;

    /**
     * Need a default constructor so that it can be support externalization
     */
    public JmsXATopicConnectionFactory() {
    }

    /**
     * Instantiate an instance of the topic connection factory with the name
     * of the class that will facilitate communication with the JmsServer
     *
     * @param name - name of the class
     * @param env - environment variables
     */
    public JmsXATopicConnectionFactory(String name, Hashtable env) {
        super(name, env);
    }

    // implementation of XATopicConnectionFactory.createXATopicConnection
    public XATopicConnection createXATopicConnection()
        throws JMSException {
        return createXATopicConnection(null, null);
    }

    // implementation of XATopicConnectionFactory.createXATopicConnection
    public XATopicConnection createXATopicConnection(String userName, String password)
        throws JMSException {
        return new JmsXATopicConnection(this, UUID.next(), userName, password);
    }
}
