/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SslHttpJmsServerStub.java,v 1.6 2003/08/07 13:32:51 tanderson Exp $
 *
 * Date             Author      Changes
 * 12 Oct 2001	    mourikis    Created
 */
package org.exolab.jms.client.http;

import java.security.Provider;
import java.security.Security;
import java.util.Hashtable;

import javax.jms.JMSException;

import org.exolab.jms.client.JmsConnectionStubIfc;


/**
 * This class is repsonsible for opening an https connection to the server
 * and creating HttpJmsConnections.
 *
 * @version     $Revision: 1.6 $ $Date: 2003/08/07 13:32:51 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         org.exolab.core.http.HttpClient
 */
public class SslHttpJmsServerStub extends HttpJmsServerStub {

    /**
     * The constructor has a collection of environment variables
     * which it uses to construct a connection to the remote
     * server
     *
     * @param env the connection properties
     * @throws JMSException if required connection properties aren't specified
     */
    public SslHttpJmsServerStub(Hashtable env) throws JMSException {
        String url = (String) env.get(HttpJmsConstants.HTTPS_SERVER_URL);
        if (url == null) {
            throw new JMSException("Required property not specified: " +
                HttpJmsConstants.HTTPS_SERVER_URL);
        }
        setURL(url);
        setLabel("SslHttpJmsServerStub");
    }

    /**
     * Construct an instance of this stub with the specified URL
     *
     * @param url url of the server hosting the servlet.
     */
    public SslHttpJmsServerStub(String url) {
        super(url);
        setLabel("SslHttpJmsServerStub");
    }

    /**
     * Create a connection to the JMS Server
     *
     * @param clientId the identity of client
     * @param username the client username
     * @param password the client password
     * @throws JMSException if the connection cannot be created
     */
    public JmsConnectionStubIfc createConnection(
        String clientId, String username, String password)
        throws JMSException {
        JmsConnectionStubIfc stub = null;

        try {
            System.setProperty("java.protocol.handler.pkgs",
                "com.sun.net.ssl.internal.www.protocol");
            Class providerClass =
                Class.forName("com.sun.net.ssl.internal.ssl.Provider");
            Provider provider = (Provider) providerClass.newInstance();
            Security.addProvider(provider);
        } catch (Exception exception) {
            throw new JMSException("Failed to initialise SSL: " + exception);
        }

        return super.createConnection(clientId, username, password);
    }

} //-- SslHttpJmsServerStub
