/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: IntravmJmsConnectionStub.java,v 1.5 2003/08/07 13:32:52 tanderson Exp $
 *
 * Date         Author  Changes
 * 05/03/2000    jima    Created
 */
package org.exolab.jms.client.intravm;

import javax.jms.JMSException;

import org.exolab.jms.client.JmsConnectionStubIfc;
import org.exolab.jms.client.JmsSessionStubIfc;
import org.exolab.jms.server.JmsServerConnection;
import org.exolab.jms.server.JmsServerConnectionManager;
import org.exolab.jms.server.JmsServerSession;


/**
 * This class is responsible for implementing the client side of a intravm
 * client-server connection for the JMS server. It is used to ovecome the
 * performance issues associated with say and RMI based client-server running
 * in the same VM. Depending on the different implementation of the client
 * server ORB (i.e RMI or CORBA) some of the smarter products will simply turn
 * a distributed request into a local client server call hence bypassing all
 * marshalling.
 * <p>
 * Generic, by implication, impliesthat we are sacrificing the performance of
 * the lower end ORB (intravm ORB) to provide a transparent framework for all
 * underlying ORBs.
 *
 * @version     $Revision: 1.5 $ $Date: 2003/08/07 13:32:52 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public class IntravmJmsConnectionStub
    implements JmsConnectionStubIfc {

    /**
     * This is a reference to the remote connection stub that is constructed during
     * object initialisation.
     */
    private JmsServerConnection _delegate = null;

    /**
     * Instantiate an instance of this class with the specified server side
     * object. All the requests for this stub are simply delegated to the
     * server skeleton
     *
     * @param       IntravmJmsServerConnectionIfc
     * @exception   JMSException
     */
    IntravmJmsConnectionStub(JmsServerConnection connection)
        throws JMSException {
        if (connection != null) {
            _delegate = connection;
        } else {
            throw new JMSException("Cannot instantiate with a null connection");
        }
    }

    // implementation of JmsConnectionStubIfc.createSession
    public JmsSessionStubIfc createSession(int ackMode, boolean transacted)
        throws JMSException {
        IntravmJmsSessionStub stub = null;

        JmsServerSession session = _delegate.createSession(ackMode, transacted);
        stub = new IntravmJmsSessionStub(session);
        session.setMessageListener(stub);

        return stub;
    }

    // implementation of JmsConnectionStubIfc.close
    public void close()
        throws JMSException {
        JmsServerConnectionManager.instance().closeConnection(
            _delegate.getID());
        _delegate.close();
        _delegate = null;
    }

    // implementation of JmsConnectionStubIfc.destroy
    public void destroy()
        throws JMSException {
        _delegate = null;
    }

    // implementation of JmsConnectionStubIfc.getConnectionId
    public String getConnectionId()
        throws JMSException {
        return _delegate.getConnectionId();
    }
}
