/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: IntravmJmsServerStub.java,v 1.7 2003/08/07 13:32:52 tanderson Exp $
 *
 * Date         Author  Changes
 * 05/03/2000   jima    Created
 */
package org.exolab.jms.client.intravm;

import java.util.Hashtable;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;

import org.exolab.jms.client.JmsConnectionStubIfc;
import org.exolab.jms.client.JmsServerStubIfc;
import org.exolab.jms.server.JmsServerConnectionManager;


/**
 * This class is a client stub facilitating communication with the server
 * in an intra-VM configuration.
 *
 * @version     $Revision: 1.7 $ $Date: 2003/08/07 13:32:52 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 */
public class IntravmJmsServerStub
    implements JmsServerStubIfc {

    /**
     * The default constructor does nothing
     */
    public IntravmJmsServerStub() {
    }

    /**
     * The constructor instantiate server based on the environment variables
     * passed in
     *
     * @param       env         environment
     */
    public IntravmJmsServerStub(Hashtable env) {
        // does nothing at the moment
    }

    /**
     * Create a connection to the JMS Server.
     *
     * @param id the identity of client
     * @param username the client username
     * @param password the client password
     * @return the connection
     * @throws JMSException if the connection cannot be created
     */
    public JmsConnectionStubIfc createConnection(String id, String username,
                                                 String password)
        throws JMSException {

        JmsConnectionStubIfc connection = new IntravmJmsConnectionStub(
            JmsServerConnectionManager.instance().createConnection(
                id, username, password));

        return connection;
    }

    // implementation of JmsServerStubIfc.setExceptionListener
    public void setExceptionListener(ExceptionListener listener) {
        // no-op for the intravm version
    }

}
