/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SslJmsServerStub.java,v 1.7 2003/08/07 13:32:54 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */
package org.exolab.jms.client.mipc;

import java.lang.reflect.Constructor;
import java.util.Hashtable;

import org.exolab.core.mipc.MultiplexConnectionIfc;


/**
 * Open an MIPC connection to the server using SSL connections.
 *
 * @version     $Revision: 1.7 $ $Date: 2003/08/07 13:32:54 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 * @see         org.exolab.jms.client.mipc.IpcJmsServerStub
 * @see         org.exolab.jms.server.mipc.IpcJmsServer
 * @see         org.exolab.jms.server.mipc.SslIpcJmsServer
 */
public class SslJmsServerStub extends IpcJmsServerStub {

    /**
     * Default constructor for serialization
     */
    public SslJmsServerStub() {
    }

    /**
     * The constructor has a collection of environment variables
     * which it uses to construct a connection to the remote
     * server
     *
     * @param           env
     */
    public SslJmsServerStub(Hashtable env) {
        super(env);
    }

    /**
     * Instantiate an instance of this stub with the specified host and
     * port numbers
     *
     * @param       host                server host address
     * @param       port                server port number
     */
    public SslJmsServerStub(String host, int port) {
        super(host, port);
    }

    /**
     * Create an SSL connection.
     *
     * @param serverAddress The address of the server.
     * @param port The port number to use.
     * @return <code>MultiplexConnectionIfc</code> The created
     * connection.
     * @throws IOException If the server fails to initialise the
     * ip service.
     */
    protected MultiplexConnectionIfc createClientConnection(
        String serverAddress, String internalServerAddress, int port)
        throws Exception {
        MultiplexConnectionIfc con = null;

        Class[] types = new Class[]{String.class, int.class};
        Class serverClass = Class.forName(
            "org.exolab.core.mipc.MultiplexSSLConnection");
        Constructor constructor = serverClass.getConstructor(types);
        try {
            Object[] args = new Object[]{serverAddress, new Integer(port)};
            con = (MultiplexConnectionIfc) constructor.newInstance(args);
        } catch (Exception err) {
            if (internalServerAddress != null) {
                Object[] args = new Object[]{internalServerAddress,
                                             new Integer(port)};
                con = (MultiplexConnectionIfc) constructor.newInstance(args);
            } else {
                // just rethrow it.
                throw err;
            }
        }
        return con;
    }

} //-- SslJmsServerStub
