/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RemoteJmsMessageListenerIfc.java,v 1.4 2003/08/07 13:32:54 tanderson Exp $
 *
 * Date         Author  Changes
 * 04/18/2000    jima    Created
 */
package org.exolab.jms.client.rmi;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

import javax.jms.Message;


/**
 * This is an RMI implementation of a RMI-based MessageListener. The message
 * listener is used by the server to push down all related messages.
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:32:54 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface RemoteJmsMessageListenerIfc
    extends Remote {

    /**
     * This method is called back by the server whenever it has a message for
     * this session
     *
     * @param message
     * @exception RemoteException
     */
    public void onMessage(Message message)
        throws RemoteException;

    /**
     * This method is called back by the server whenever it has one or more
     * messages for this session
     *
     * @param messages - vector of messages
     * @exception RemoteException
     */
    public void onMessages(Vector messages)
        throws RemoteException;

    /**
     * Notify the client that a message is available for it on the server
     *
     * @param clientId - the id of the client to notify
     * @exception RemoteException
     */
    public void onMessageAvailable(long clientId)
        throws RemoteException;
}
