/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RmiJmsMessageListener.java,v 1.4 2003/08/07 13:32:54 tanderson Exp $
 *
 * Date         Author  Changes
 * 04/18/2000    jima    Created
 */
package org.exolab.jms.client.rmi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import javax.jms.JMSException;
import javax.jms.Message;

import org.exolab.jms.client.JmsMessageListener;


/**
 * This is an implementation of the RemoteJmsMessageListenerIfc interface. It
 * is instantiayed with an instance of a JmsSession object and delegates the
 * request down to it.
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:32:54 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public class RmiJmsMessageListener
    extends UnicastRemoteObject
    implements RemoteJmsMessageListenerIfc {

    /**
     * This is the message listener for JMS messages. The MessageListener is then
     * responsible for delivering it to the actual consumer. The listener is
     * initialised during object construction time
     */
    private JmsMessageListener _delegate = null;

    /**
     * Instantiate an instance of this class with a non null listener. If a
     * null MessageListener is specified then RemoteException is thrown.
     *
     * @param       listener
     * @exception   JMSException
     * @exception   RemoteException
     */
    public RmiJmsMessageListener(JmsMessageListener listener)
        throws JMSException, RemoteException {
        if (listener != null) {
            _delegate = listener;
        } else {
            throw new JMSException("Cannot create object with null listener");
        }
    }

    // implementation of RemoteJmsMessageListenerIfc.onMessage
    public void onMessage(Message message)
        throws RemoteException {
        _delegate.onMessage(message);
    }

    // implementation of RemoteJmsMessageListenerIfc.onMessages
    public void onMessages(Vector messages)
        throws RemoteException {
        _delegate.onMessages(messages);
    }

    // implementation of RemoteJmsMessageListenerIfc.onMessageAvailable
    public void onMessageAvailable(long clientId)
        throws RemoteException {
        _delegate.onMessageAvailable(clientId);
    }

}
