/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: EventHandler.java,v 1.3 2003/08/17 01:32:22 tanderson Exp $
 *
 * Date         Author  Changes
 * 07/27/00	    jima    Created
 */

package org.exolab.jms.events;

import java.io.Serializable;

import org.exolab.core.foundation.HandleIfc;


/**
 * This class defines an object that receives event notification from an
 * {@link EventManager}. It also defines a method to return a
 * <code>HandleIfc</code> to the event handler, making it possible to be 
 * resolved at some future time (i.e across EventManager invocations)
 *
 * @version     $Revision: 1.3 $ $Date: 2003/08/17 01:32:22 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 * @see         EventManager
 * @see         Event
 */
public interface EventHandler
    extends Serializable {

    /**
     * Handle the specified event, generated by the {@link EventManager}.
     * The event handler pressumably knows how to handle the callback object
     * and should cast it appropriately before using it.
     * <p>
     * This methodshould not throw any exceptions
     *
     * @param       event           the event to handle
     * @param       callback        the callback object
     * @param       time            the absolute time, in ms, that the event
     *                              fired.
     */
    void handleEvent(int event, Object callback, long time);

    /**
     * Return a handle to this event handler for the purpose of storing and
     * restoring it. If you want event handlers to persist across subsequent
     * invocations of the {@link EventManager} then you must support this
     * method, otherwise this method can simply return null.
     *
     * @return      HandleIfc       return a reference to the handle.
     */
    HandleIfc getHandle();
}
