/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: InitialContextFactory.java,v 1.3 2003/08/17 01:32:23 tanderson Exp $
 */
package org.exolab.jms.jndi;

import java.util.Hashtable;

import javax.naming.ConfigurationException;
import javax.naming.Context;
import javax.naming.NamingException;

import org.exolab.jms.jndi.http.HttpJndiInitialContextFactory;
import org.exolab.jms.jndi.http.SslHttpJndiInitialContextFactory;
import org.exolab.jms.jndi.intravm.IntravmJndiInitialContextFactory;
import org.exolab.jms.jndi.mipc.IpcJndiInitialContextFactory;
import org.exolab.jms.jndi.rmi.RmiJndiInitialContextFactory;


/**
 * A factory that creates an initial context to an embedded OpenJMS
 * JNDI provider.
 *
 * @version     $Revision: 1.3 $ $Date: 2003/08/17 01:32:23 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 */
public class InitialContextFactory
    implements javax.naming.spi.InitialContextFactory {

    /**
     * Creates an initial context for beginning name resolution, based on
     * the {@link Context#PROVIDER_URL} attribute.
     *
     * @param environment the environment specifying information to be used in
     * the creation of the initial context.
     * @return an initial context
     * @throws NamingException if the initial context cannot be created
     */
    public Context getInitialContext(Hashtable environment)
        throws NamingException {
        if (environment == null) {
            throw new ConfigurationException(
                "Cannot connect to JNDI provider - environment not set");
        }
        String url = (String) environment.get(Context.PROVIDER_URL);
        if (url == null) {
            throw new ConfigurationException(
                "Connect connect to JNDI provider - " +
                Context.PROVIDER_URL + " not set ");
        }

        javax.naming.spi.InitialContextFactory factory;

        if (url.startsWith("rmi")) {
            factory = new RmiJndiInitialContextFactory();
        } else if (url.startsWith("tcp")) {
            factory = new IpcJndiInitialContextFactory();
        } else if (url.startsWith("https")) {
            factory = new SslHttpJndiInitialContextFactory();
        } else if (url.startsWith("http")) {
            factory = new HttpJndiInitialContextFactory();
        } else if (url.startsWith("embedded")) {
            factory = new IntravmJndiInitialContextFactory();
        } else {
            throw new ConfigurationException(
                "Cannot connect to JNDI provider - invalid URL: " + url);
        }

        return factory.getInitialContext(environment);
    }

} //-- InitialContextFactory
