/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: JndiNameParser.java,v 1.4 2003/08/07 13:32:57 tanderson Exp $
 *
 * Date         Author  Changes
 * 3/14/2000    jima    Created
 */
package org.exolab.jms.jndi;


import java.util.Properties;

import javax.naming.CompoundName;
import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingException;


/**
 * implementation of the NameParser interface for the JndiNameParser. The
 * JndiNameParser uses the dot notation to define names
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:32:57 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         javax.naming.NameParser
 **/
public class JndiNameParser
    implements NameParser {

    private static final Properties syntax__ = new Properties();

    static {
        syntax__.put("jndi.syntax.direction", "left_to_right");
        syntax__.put("jndi.syntax.separator", "@@@@");
        syntax__.put("jndi.syntax.ignorecase", "false");
        syntax__.put("jndi.syntax.escape", "\\");
        syntax__.put("jndi.syntax.beginquote", "'");
    }

    /**
     * Parse the specified name using the class defined syntax. If there is
     * a problem parsing the name then throw NamingException
     *
     * @param       name            name to parse
     * @return      Name            constructed compound name
     * @exception   NamingException
     */
    public Name parse(String name)
        throws NamingException {
        return new CompoundName(name, syntax__);
    }
}
