/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: AdminConnection.java,v 1.4 2003/08/17 01:32:23 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */


package org.exolab.jms.jndiadministration;

import java.util.Enumeration;


/**
 * The abstract class all AdminConnection objects must inherit from.
 * Currently there are only two object types. OfflineConnection, for objects
 * that directly connect to and use the persistency mechanism, and
 * OnlineConnection objects that connect to an OpenJMSServer for all
 * their requests.
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/17 01:32:23 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         OfflineConnection
 * @see		OnlineConnection
 */
public abstract class AdminConnection {

    protected static AdminConnection instance_ = null;

    /**
     * Returns the one and only instance of the connection object.
     *
     * @return AdminConnection The one and only instance.
     *
     */
    public static AdminConnection instance() {
        return instance_;
    }

    /**
     * Close the connection.
     *
     */
    public abstract void close();


    /**
     * Get an enumerated list of all the Contexts
     *
     * @return 	Enumeration The list of contexts
     *
     */
    public abstract Enumeration getAllContexts(String name);


    /**
     * Return the object associated with this context.
     * if none exists return null.
     *
     * @param context The context name
     * @return Object	The object for this context.
     *
     */
    public abstract Object lookup(String context);


    /**
     * Create a new context with the given name.
     *
     * @param name The new context name.
     * @exception NamingException If the context cannot be created.
     *
     */
    public abstract void createContext(String name)
        throws javax.naming.NamingException;


    /**
     * Destroy context with the given name.
     *
     * @param name The new context name.
     * @exception NamingException If the context cannot be created.
     *
     */
    public abstract void destroyContext(String name)
        throws javax.naming.NamingException;


    /**
     * Rename context with the given name.
     *
     * @param oldName The old context name
     * @param newName The new context name
     * @exception NamingException If the context cannot be created.
     *
     */
    public abstract void renameContext(String oldName, String newName)
        throws javax.naming.NamingException;


    /**
     * Rebind the context with the given object.
     *
     * @param name The context name
     * @param ob The object to bind in this context
     * @exception NamingException If the context cannot be created.
     *
     */
    public abstract void rebind(String name, Object ob)
        throws javax.naming.NamingException;

} // End AdminConnection
