/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ObjectDialog.java,v 1.2 2003/08/07 13:32:59 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */

package org.exolab.jms.jndiadministration;

import javax.swing.JFrame;
import javax.swing.JOptionPane;


/**
 * A simple dialog to collect information for creating Contexts and objects
 *
 *
 * @version     $Revision: 1.2 $ $Date: 2003/08/07 13:32:59 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         org.exolab.jms.jndiadministration.AdminMgr
 *
 */

public class ObjectDialog {

    // The name chosen for this object
    private String name_;

    // The dialogs parent
    private JFrame parent_;

    // The two possible states of theis dialog.
    final static public int CANCELED = 1;
    final static public int CONFIRMED = 2;

    // The command status used to shutdown this window.
    private int status_;

    // The one and only instance of this object.
    static private ObjectDialog instance_;

    /**
     * Creates new form ObjectDialog
     *
     * @param parent The parent form.
     */
    public ObjectDialog(JFrame parent) {
        parent_ = parent;
    }

    /**
     * Get the one and only instance of this dialog. The dialog must first
     * have been created with the create call below.
     *
     * @return ObjectDialog the one and only instance
     *
     */
    public static ObjectDialog instance() {
        return instance_;
    }


    /**
     * Create the one and only instance of the Object Dialog.
     *
     * @param parent the parent of this dialog
     * @return ObjectDialog the one and only instance
     *
     */
    public static ObjectDialog create(JFrame parent) {
        if (instance_ == null) {
            instance_ = new ObjectDialog(parent);
        }
        return instance_;
    }


    /**
     * Display the dialog.
     *
     * @param displayString The display string
     * @param name The dialog name
     *
     */
    public void display(String displayString, String name) {
        name_ = JOptionPane.showInputDialog
            (parent_, displayString, name, JOptionPane.PLAIN_MESSAGE);
        if (name_ == null || name_.equals("")) {
            status_ = CANCELED;
        } else {
            status_ = CONFIRMED;
        }
    }

    /**
     * Whether this dialog was confirmed or canceled.
     *
     * @return boolena true if the OK button was pressed.
     *
     */
    public boolean isConfirmed() {
        return status_ == CONFIRMED;
    }

    /**
     * Get the name selected for this Object
     *
     * @return String The name entered by the user
     */
    public String getName() {
        return name_;
    }

} // End ObjectDialog
