/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: OpenJMSObject.java,v 1.3 2003/08/17 01:32:23 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */


package org.exolab.jms.jndiadministration;

import java.awt.Rectangle;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;


/**
 * This class controls all dispay characteristics and menus related to an
 * object. Currently this is nothing more than just displaying the Object name.
 *
 * @version     $Revision: 1.3 $ $Date: 2003/08/17 01:32:23 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         AdminMgr
 * @see		OpenJMSContext
 * @see	        OpenJMSNode
 */
public class OpenJMSObject extends DefaultMutableTreeNode
    implements OpenJMSNode {

    // The unique object name.
    private String objectName_;

    // A reference to the tree this node belongs to.
    static private JTree tree_ = null;

    /**
     * The constructor gets its unique name for this object plus a
     * reference to its parent tree.
     *
     * @param objectName This objects unique name.
     * @param tree The parent tree this object belons to.
     *
     */
    public OpenJMSObject(String objectName, JTree tree) {
        objectName_ = objectName;
        tree_ = tree;
    }

    /**
     * Objects are always leaves.
     *
     * @return boolean Always returns false.
     *
     */
    public boolean getAllowsChildren() {
        return false;
    }


    /**
     * All objects are leaves in the tree for this release.
     *
     * @return boolean Always returns true.
     *
     */
    public boolean isLeaf() {
        return true;
    }


    /**
     * This method is defined by the interface, but is not required by
     * objects for this release.
     *
     */
    public void update() {
        // do nothing for the moment
    }

    /**
     * This node has been right clicked. The locations of this node is given
     * by the loc object.
     * There are no menus for Objects yet.
     *
     * @param The location of this Object node.
     *
     */
    public void displayCommands(Rectangle loc) {
//  		double x;
//  		double y;

//  		x = loc.getX();
//  		y = loc.getY();
//  		y += loc.getHeight();

//  		commands_.show(tree_, (int)x, (int)y);
    }


    /**
     * The Objects name.
     *
     * @return String the Object name.
     *
     */
    public String toString() {
        return objectName_;
    }

    /**
     * This node has changed. Inform the parent tree that it needs to be
     * re-drawn.
     *
     */
    private void refresh() {
        DefaultTreeModel model = (DefaultTreeModel) tree_.getModel();
        model.nodeStructureChanged((DefaultMutableTreeNode) this);
    }

} // End OpenJMSObject
