/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: LeaseIfc.java,v 1.2 2003/08/07 13:33:00 tanderson Exp $
 *
 * Date         Author  Changes
 * 2/29/2000    jima    Created
 */
package org.exolab.jms.lease;


/**
 * A lease is used to track message exipration. It contains the object to be
 * leased, the duration and the object to callback if or when the lease expirex
 * <p>
 * Since leases may be persistent we need to store the absolute expiry time so
 * that when the service is restarted it can correctly expire messages
 *
 * @version     $Revision: 1.2 $ $Date: 2003/08/07 13:33:00 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface LeaseIfc {

    /**
     * Return the absolute expiry time of this lease
     *
     * @return      long
     */
    public long getExpiryTime();

    /**
     * Return the orginal duration of the lease. The returned value is
     * in milliseconds
     *
     * @return      long
     */
    public long getDuration();

    /**
     * Return the time remaining on the lease in milliseconds
     *
     * @return      long
     */
    public long getRemainingTime();

    /**
     * Return the leased object
     *
     * @return      Object
     */
    public Object getLeasedObject();

    /**
     * Return the class of leased object
     *
     * @return      Class
     */
    public Class getLeasedObjectType();
}

