/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: DestinationCacheEventListener.java,v 1.4 2003/08/17 01:32:24 tanderson Exp $
 *
 * Date         Author  Changes
 * 3/1/2001     jima    Created
 */
package org.exolab.jms.messagemgr;

import java.sql.Connection;

import org.exolab.jms.message.MessageImpl;
import org.exolab.jms.persistence.PersistenceException;


/**
 * A DestinationCacheEventListener responds to events generated by a
 * {@link DestinationCache}.
 * It gets notified when a message is added or removed for a particular 
 * destination.
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/17 01:32:24 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 */
public interface DestinationCacheEventListener {

    /**
     * This event is called when a non-persistent message is added to the
     * DestinationCache.
     *
     * @param message - message added to cache
     */
    public boolean messageAdded(MessageImpl message);

    /**
     * This event is called when a message is removed from the DestinationCache.
     *
     * @param message - message removed from cache
     */
    public boolean messageRemoved(MessageImpl message);

    /**
     * This event is called when a persistent message is added to the
     * DestinationCache.
     *
     * @param connection - the database connection
     * @param message - message added to cache
     * @throws PersistenceException - if there is a persistence related problem
     */
    public boolean persistentMessageAdded(Connection connection,
                                          MessageImpl message)
        throws PersistenceException;

    /**
     * This event is called when a message is removed from the 
     * DestinationCache.
     *
     * @param connection - the database connection
     * @param message - message to remove from cache
     * @throws PersistenceException - if there is a persistence related problem
     */
    public boolean persistentMessageRemoved(Connection connection,
                                            MessageImpl message)
        throws PersistenceException;
}

