/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: InternalMessageListener.java,v 1.4 2003/08/07 13:33:03 tanderson Exp $
 *
 * Date         Author  Changes
 * 09/12/2001   jima    Created
 */
package org.exolab.jms.messagemgr;

import java.util.Vector;

import org.exolab.jms.message.MessageHandle;


/**
 * This is the server side implementation of a message listener
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:33:03 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface InternalMessageListener {

    /**
     * Send the specified message to the client. If the ack paramter
     * is set then comply to the acking policy of the session. If this
     * parameter is not set then ignore the acking policy. The later
     * mode is used by the queue browser, when it sends messages
     *
     * @param message - message handle
     * @param ack - false ignores the acking policy
     * @throws Exception - propagate exceptions to client
     */
    public void onMessage(MessageHandle message, boolean ack)
        throws Exception;

    /**
     * Send the specified collection of messages to the client
     *
     * @param messages - collection of MessageHandle objects
     * @throws Exception - propagate exceptions to client
     */
    public void onMessages(Vector messages)
        throws Exception;

    /**
     * Inform the session that there is a message available
     * for the message consumer with the specified identity
     *
     * @param clientId - the identity of the client
     */
    public void onMessageAvailable(long clientId)
        throws Exception;
}

