/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: PersistentString.java,v 1.7 2003/08/17 01:32:25 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */


package org.exolab.jms.persistence;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.exolab.core.foundation.PersistentObject;


/**
 * A simple helper class to wrap strings, and make object referenses easier.
 *
 * @version     $Revision: 1.7 $ $Date: 2003/08/17 01:32:25 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 */
public class PersistentString
    extends PersistentObject
    implements Externalizable {

    // Used for serialization
    static final long serialVersionUID = 1;


    // The String holder
    private String id_;

    /**
     * The default constructor initialises an empty String
     *
     */
    public PersistentString() {
        super();
        id_ = "";
    }


    /**
     * Initialise to the given String value
     *
     * @param id The string to assign
     *
     */
    public PersistentString(String id) {
        super();
        id_ = id;
    }


    /**
     * Assign the given String value
     *
     * @param id The string to assign
     *
     */
    public void update(String id) {
        id_ = id;
    }


    /**
     * Return the internal String
     *
     * @return String The internal String
     *
     */
    public String toString() {
        return id_;
    }


    /**
     * Return the length of the internal String
     *
     * @return int The internal String length
     *
     */
    public int length() {
        return (id_ == null ? 0 : id_.length());
    }

    // override Object.equals
    public boolean equals(Object obj) {

        boolean result = false;

        if ((obj instanceof PersistentString) &&
            (id_.equals(((PersistentString) obj).id_))) {
            result = true;
        }

        return result;
    }

    // override Object.hashCode
    public int hashCode() {
        return id_.hashCode();
    }

    // implementation of Externalizable.writeExternal
    public void writeExternal(ObjectOutput stream)
        throws IOException {
        stream.writeLong(serialVersionUID);
        stream.writeObject(id_);
        super.writeExternal(stream);
    }

    // implementation of Externalizable.writeExternal
    public void readExternal(ObjectInput stream)
        throws IOException, ClassNotFoundException {
        long version = stream.readLong();
        if (version == serialVersionUID) {
            id_ = (String) stream.readObject();
            super.readExternal(stream);
        } else {
            throw new IOException("PersistentString with version " +
                version + " is not supported.");
        }
    }
} // End PersistentString
