/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2004 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: EmbeddedJmsServer.java,v 1.18 2004/01/03 04:00:56 tanderson Exp $
 *
 * Date         Author  Changes
 * 05/04/2000   jima    Created
 */
package org.exolab.jms.server;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.exolab.core.service.ServiceManager;
import org.exolab.jms.config.Configuration;


/**
 * This class is used to embed the JMS Server in another component, e.g an
 * EJB or Servlet container etc. It implements Runnable and should
 * be instantiated in its own thread.
 * <code>
 *  new Thread(new EmbeddedJmsServer("jms_config.xml"))
 * </code>
 *
 * @version     $Revision: 1.18 $ $Date: 2004/01/03 04:00:56 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         java.lang.Runnable
 */
public class EmbeddedJmsServer
    extends JmsServer
    implements Runnable {

    /**
     * The logger
     */
    private static final Log _log = LogFactory.getLog(EmbeddedJmsServer.class);


    /**
     * Construct a new <code>EmbeddedJmsServer</code>
     *
     * @param config the server configuration
     * @throws ServerException if the server cannot be created
     */
    public EmbeddedJmsServer(Configuration config) throws ServerException {
        super(config);
    }

    /**
     * Construct a new <code>EmbeddedJmsServer</code>
     *
     * @param file configuration file name
     * @throws ServerException if the server cannot be created
     */
    public EmbeddedJmsServer(String file) throws ServerException {
        super(file);
    }

    /**
     * This method calls the init method on the serverObj_.It assumes that the
     * serverObj, has been correctly initialised, which is the responsibility
     * of the constructor. If it fails to initialise the server then it throws
     * FailedToInitialiseServerException exception
     *
     */
    public void run() {
        try {
            init();
        } catch (Exception exception) {
            _log.error(exception, exception);
        }
        // @todo - we still need to do something about correctly stopping and
        // pausing the application
    }

    /**
     * Return an instance of the service manager
     *
     * @return ServiceManager
     */
    public ServiceManager getServiceManager() {
        return _services;
    }

} //-- EmbeddedJmsServer
