/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: NamingHelper.java,v 1.4 2003/08/17 01:32:26 tanderson Exp $
 */
package org.exolab.jms.server;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.exolab.jms.config.Configuration;
import org.exolab.jms.config.JndiConfiguration;
import org.exolab.jms.config.Property;


/**
 * This class provides helper methods to access the name service's initial
 * context.
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/17 01:32:26 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         EmbeddedNameService
 */
public class NamingHelper {

    /**
     * Return the initial context. <br/>
     * If an embedded JNDI service is configured, then return its initial
     * context, else return the initial context of the external JNDI provider.
     *
     * @param config the server configuration
     * @return the initial context
     * @throws NamingException if a naming error occurs
     */
    public static Context getInitialContext(Configuration config)
        throws NamingException {
        Context initial = null;

        if (config.getServerConfiguration().getEmbeddedJNDI()) {
            initial = EmbeddedNameService.getInstance().getInitialContext();
        } else {
            Hashtable environment = new Hashtable();
            Property[] properties =
                config.getJndiConfiguration().getProperty();
            for (int index = 0; index < properties.length; ++index) {
                environment.put(properties[index].getName(),
                    properties[index].getValue());
            }
            initial = new InitialContext(environment);
        }
        return initial;
    }

} //-- NamingHelper
