/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 *
 * $Id: HttpJmsSessionPinger.java,v 1.5 2003/08/17 01:32:26 tanderson Exp $
 *
 * Date				Author  Changes
 * Wed 31 Oct 2001	mourikis    Created
 */


package org.exolab.jms.server.http;

import org.exolab.jms.config.Configuration;
import org.exolab.jms.config.ConfigurationManager;
import org.exolab.jms.config.HttpConfiguration;


/**
 * A simpler pinger, which runs in its own thread and is responsible for
 * pinging a client session, if no messages have been sent to the client with a
 * timeout period. Package protected.
 *
 * @version     $Revision: 1.5 $ $Date: 2003/08/17 01:32:26 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 * @see         HttpJmsSessionSender
 */
public class HttpJmsSessionPinger implements Runnable {

    // The session we require to ping.
    private HttpJmsSessionSender session_ = null;

    // The session is closed, finish pinging.
    private boolean finished_ = false;

    // A message was sent during the timeout period, reset the clock.
    private boolean reset_ = false;

    // The timeout period for pings.
    private int timeout_;

    /**
     * Construct pinger, with a reference to appropriate session.
     *
     * @param session The session we are interested in pinging.
     *
     */
    HttpJmsSessionPinger(HttpJmsSessionSender session) {
        session_ = session;
        HttpConfiguration config =
            ConfigurationManager.getConfig().getHttpConfiguration();
        timeout_ = config.getClientPingInterval() * 1000;
        if (timeout_ == 0) {
            timeout_ = 60 * 1000;
        }
    }

    /**
     * Reset the timeout, client has sent a message.
     */
    void reset() {
        reset_ = true;
    }


    /**
     * Session has closed. Set the flag to stop pining.
     */
    void close() {
        finished_ = true;
    }

    /**
     * The main loop of the thread which does the pinging.
     */
    public void run() {
        while (!finished_) {
            try {
                reset_ = false;
                Thread.sleep(timeout_);
                if (!finished_ && !reset_) {
                    session_.ping();
                }
            } catch (org.exolab.jms.server.ClientDisconnectionException e) {
                session_.close();
            } catch (InterruptedException ignore) {
            }
        }
        session_ = null;
    }


} // End HttpJmsSessionPinger
