/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: HttpsJmsServer.java,v 1.2 2003/08/07 13:33:08 tanderson Exp $
 */

package org.exolab.jms.server.http;

import java.security.Provider;
import java.security.Security;

import org.exolab.jms.client.http.HttpJmsConstants;
import org.exolab.jms.config.ConfigurationManager;
import org.exolab.jms.config.HttpConfigurationType;
import org.exolab.jms.config.types.SchemeType;
import org.exolab.jms.server.ServerException;


/**
 * This class enables clients to connect to the JMS server via
 * a HTTP server.
 * It must be used in conjunction with the TCP connector.
 *
 * @version     $Revision: 1.2 $ $Date: 2003/08/07 13:33:08 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 */
public class HttpsJmsServer extends HttpJmsServer {

    /**
     * Construct a new <code>HttpsJmsServer</code>
     */
    public HttpsJmsServer() {
    }

    /**
     * Initialise the server
     *
     * @throws ServerException if SSL support can't be initialised
     */
    public void init() throws ServerException {

        // Initialise all the stuff required to connect back
        // to a client using an https url. Set any proxies necessary
        // to get through firewall.
        HttpConfigurationType config =
            ConfigurationManager.getConfig().getHttpsConfiguration();

        if (config.getProxyHost() != null) {
            System.setProperty("https.proxyHost", config.getProxyHost());
        }
        if (config.hasProxyPort()) {
            System.setProperty("https.proxyPort",
                Integer.toString(config.getProxyPort()));
        }

        // Set security properties.
        System.setProperty("java.protocol.handler.pkgs",
            "com.sun.net.ssl.internal.www.protocol");
        try {
            Class providerClass =
                Class.forName("com.sun.net.ssl.internal.ssl.Provider");
            Provider provider = (Provider) providerClass.newInstance();
            Security.addProvider(provider);
        } catch (Exception exception) {
            throw new ServerException("Failed to initialise SSL support",
                exception);
        }
    }

    /**
     * Returns the connector scheme for this server.
     */
    protected SchemeType getScheme() {
        return SchemeType.HTTPS;
    }

    /**
     * Returns the HTTP configuration
     */
    protected HttpConfigurationType getHttpConfiguration() {
        return ConfigurationManager.getConfig().getHttpsConfiguration();
    }

    /**
     * Returns the HTTP server URL property name, used in the construction
     * of connection factories
     */
    protected String getHttpServerProperty() {
        return HttpJmsConstants.HTTPS_SERVER_URL;
    }

} //--HttpsJmsServer
