/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: IntravmJndiServer.java,v 1.2 2003/08/07 13:33:09 tanderson Exp $
 *
 * Date         Author  Changes
 * 05/05/2000    jima    Created
 * 10/9/2000     mourikis Added JndiFactory and removed Jdbm peristency.
 */
package org.exolab.jms.server.intravm;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;

import org.exolab.jms.jndi.JndiServerIfc;


/**
 * This class enables clients to interface to the embedded JNDI provider
 * using the embedded connector
 *
 * @version     $Revision: 1.2 $ $Date: 2003/08/07 13:33:09 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 */
public class IntravmJndiServer
    implements JndiServerIfc, InitialContextFactory {

    /**
     * The singleton instance
     */
    private static IntravmJndiServer _instance;

    /**
     * The root context
     */
    private Context _context;

    /**
     * Construct a new <code>IntravmJndiServer</code>
     */
    public IntravmJndiServer(Context context) {
        _context = context;
        _instance = this;
        // bad bad bad - will die when the new connector arch is integrated
    }

    /**
     * Initialise the JNDI service
     */
    public void init() {
        // no-op
    }

    /**
     * Creates an initial context for beginning name resolution
     *
     * @param environment the environment specifying information to be used in
     * the creation of the initial context.
     * @return an initial context
     */
    public Context getInitialContext(Hashtable environment)
        throws NamingException {
        return (Context) _context.lookup(""); // copy the initial context
    }

    /**
     * Returns the singleton instance
     */
    public static IntravmJndiServer getInstance() {
        return _instance;
    }

}
