/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: IpcJmsAdminConnection.java,v 1.10 2003/08/17 01:32:26 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date       jimm    Created
 */

package org.exolab.jms.server.mipc;

import java.io.Serializable;
import java.util.Vector;

import javax.jms.JMSException;
import javax.jms.JMSSecurityException;

import org.exolab.core.ipc.NotifierIfc;
import org.exolab.jms.server.AdminConnection;
import org.exolab.jms.server.AdminConnectionManager;


/**
 * This is the server side receiver for admin requests.
 *
 * @version     $Revision: 1.10 $ $Date: 2003/08/17 01:32:26 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @see         org.exolab.jms.administration.mipc.IpcJmsAdminConnection
 */
public class IpcJmsAdminConnection implements NotifierIfc {

    /**
     * Default constructor.
     *
     */
    public IpcJmsAdminConnection() {
    }


    /**
     * A new request has been received.
     * Carry out the request, and pass back any relevent data.
     *
     * @param ob The data received,
     * @param id The id of the calling connection, not used by the client.
     * @return Object Return any requested result. This must never be null.
     *
     */
    public Serializable notify(Object ob, String id) {
        Vector v = (Vector) ob;
        String func = (String) v.get(1);
        Serializable result = null;
        String identifierId = (String) v.get(2);

        if (func.equals("createConnection")) {
            result = createConnection(id, (String) v.get(3), (String) v.get(4));
        } else if (func.equals("addDurableConsumer")) {
            result = addDurableConsumer((String) v.get(3), (String) v.get(4),
                (String) v.get(2));
        } else if (func.equals("removeDurableConsumer")) {
            result = removeDurableConsumer((String) v.get(3),
                (String) v.get(2));
        } else if (func.equals("durableConsumerExists")) {
            result = durableConsumerExists((String) v.get(3),
                (String) v.get(2));
        } else if (func.equals("getDurableConsumers")) {
            result = getDurableConsumers((String) v.get(3), (String) v.get(2));
        } else if (func.equals("unregisterConsumer")) {
            result = unregisterConsumer((String) v.get(3), (String) v.get(2));
        } else if (func.equals("isConnected")) {
            result = isConnected((String) v.get(3), (String) v.get(2));
        } else if (func.equals("getAllDestinations")) {
            result = getAllDestinations((String) v.get(2));
        } else if (func.equals("addDestination")) {
            result = addDestination((String) v.get(3), (Boolean) v.get(4),
                (String) v.get(2));
        } else if (func.equals("removeDestination")) {
            result = removeDestination((String) v.get(3), (String) v.get(2));
        } else if (func.equals("destinationExists")) {
            result = destinationExists((String) v.get(3), (String) v.get(2));
        } else if (func.equals("getDurableConsumerMessageCount")) {
            result = getDurableConsumerMessageCount((String) v.get(3),
                (String) v.get(4), (String) v.get(2));
        } else if (func.equals("getQueueMessageCount")) {
            result = getQueueMessageCount((String) v.get(3), (String) v.get(2));
        } else if (func.equals("stopServer")) {
            result = stopServer((String) v.get(2));
        } else if (func.equals("addUser")) {
            result = addUser((String) v.get(3), (String) v.get(4),
                (String) v.get(2));
        } else if (func.equals("changePassword")) {
            result = changePassword((String) v.get(3), (String) v.get(4),
                (String) v.get(2));
        } else if (func.equals("removeUser")) {
            result = removeUser((String) v.get(3), (String) v.get(2));
        } else if (func.equals("getAllUsers")) {
            result = getAllUsers((String) v.get(2));
        } else if (func.equals("purgeMessages")) {
            result = purgeMessages((String) v.get(2));
        } else {
            String st = ("Unknown request received: " + func);
            System.err.println(st);
            result = pack(new Boolean(false), st);
        }

        return result;
    }

    /**
     * The connection has been broken.
     *
     * @param id   the unique identifier of this connection.
     *
     */
    public void disconnection(String id) {
    }

    /**
     * A new connection request has been made.
     *
     * @param ipcId The unique IPC id of this connection.
     * @param username the client's username
     * @param password the client's password
     * @return Vector The result of the request.
     */
    private Vector createConnection(String ipcId,
                                    String username,
                                    String password) {
        Vector result;

        try {
            AdminConnection connection =
                AdminConnectionManager.instance().createConnection(
                    ipcId, username, password);
            result = pack(Boolean.TRUE, connection.getIdentifierId());
        } catch (JMSException exception) {
            result = pack(Boolean.FALSE, exception);
        }

        return result;
    }

    /**
     * Add a durable consumer to for a topic
     *
     * @param topic         the name of the topic destination
     * @param name          consumer name
     * @param identifierId  the unique identifier for this connection
     * @return Vector      The result of the operation
     */
    private Vector addDurableConsumer(String topic, String name,
                                      String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).addDurableConsumer(topic, name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Remove the specified durable consumer
     *
     * @param name - the consumer name
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     */
    private Vector removeDurableConsumer(String name, String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).removeDurableConsumer(name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Determine whether the specified durable consumer exists
     *
     * @param name - the consumer name
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     */
    private Vector durableConsumerExists(String name, String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).durableConsumerExists(name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Return a list of durable consumers for the specified topic
     *
     * @param topic - the queue/topic name
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     *
     */
    private Vector getDurableConsumers(String topic, String identifierId) {
        Vector consumers;

        try {
            consumers =
                getAdminConnection(identifierId).getDurableConsumers(topic);
            return pack(Boolean.TRUE, consumers);
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Deactivate the specified durable consumer
     *
     * @param name - the name of the durable consumer
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     */
    private Vector unregisterConsumer(String name, String identifierId) {
        boolean result;

        try {
            result = getAdminConnection(identifierId).unregisterConsumer(name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Check to see if the given consumer is currently connected to the
     * OpenJMSServer. This is only valid when in online mode.
     *
     * @param name - the unique consumer identifier.
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     */
    private Vector isConnected(String name, String identifierId) {
        boolean result;

        try {
            result = getAdminConnection(identifierId).isConnected(name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Add a new destination
     *
     * @param destination - the queue/topic name
     * @param queue - true if this is a queue, false is a topic
     * @param identifierId  the unique identifier for this connection
     * @return Vector - the result of the operation
     */
    private Vector addDestination(String destination, Boolean queue,
                                  String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).addDestination(destination, queue);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Destroy all references to a queue/topic. Remove all messages and
     * Consumer registrations.
     *
     * @param destination - the queue/topic name
     * @param identifierId  the unique identifier for this connection
     * @return Vector - result of the operation
     */
    private Vector removeDestination(String destination, String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).removeDestination(destination);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Check if the specified destination exists.
     *
     * @param name - the destination name to check
     * @param identifierId  the unique identifier for this connection
     * @return Vector - result of the operation
     */
    private Vector destinationExists(String name, String identifierId) {
        boolean result;

        try {
            result = getAdminConnection(identifierId).destinationExists(name);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Get a list of all the destinations
     *
     * @param identifierId  the unique identifier for this connection
     * @return Vector    The result of the operation
     */
    private Vector getAllDestinations(String identifierId) {
        Vector result;

        try {
            result = getAdminConnection(identifierId).getAllDestinations();
            return pack(Boolean.TRUE, result);
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Return the number of messages for a specific durable consumer
     *
     * @param topic         topic name
     * @param name          consumer name
     * @param identifierId  the unique identifier for this connection
     * @return Vector       The result of the operation
     */
    private Vector getDurableConsumerMessageCount(String topic, String name,
                                                  String identifierId) {
        int result;

        try {
            result = getAdminConnection(identifierId).
                getDurableConsumerMessageCount(topic, name);
            return pack(Boolean.TRUE, new Integer(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Return the number of messages in the given queue
     *
     * @param queue             queue name
     * @param identifierId  the unique identifier for this connection
     * @return Vector           The result of the operation
     */
    private Vector getQueueMessageCount(String queue, String identifierId) {
        int result;

        try {
            result =
                getAdminConnection(identifierId).getQueueMessageCount(queue);
            return pack(Boolean.TRUE, new Integer(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Remove all processed messages from the database.
     *
     * @param identifierId  the unique identifier for this connection
     * @return Vector          The result of the operation
     */
    private Vector purgeMessages(String identifierId) {
        int result;

        try {
            result = getAdminConnection(identifierId).purgeMessages();
            return pack(Boolean.TRUE, new Integer(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Send a stop request to the OpenJMSServer.
     *
     * @param identifierId  the unique identifier for this connection
     * @return Vector              The result of the operation
     */
    private Vector stopServer(String identifierId) {
        try {
            getAdminConnection(identifierId).stopServer();
            return pack(Boolean.TRUE, Boolean.TRUE);
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Add a user with the specified name
     *
     * @param username    the users name
     * @param password    the users password
     * @param identifierId  the unique identifier for this connection
     * @return Vector     The result of the operation
     */
    private Vector addUser(String username, String password,
                           String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).addUser(username, password);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Change password for the specified user
     *
     * @param username    the users name
     * @param password    the users password
     * @param identifierId  the unique identifier for this connection
     * @return Vector     The result of the operation
     */
    private Vector changePassword(String username, String password,
                                  String identifierId) {
        boolean result;

        try {
            result =
                getAdminConnection(identifierId).changePassword(username, password);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Remove the specified user
     *
     * @param username    the users name
     * @param identifierId  the unique identifier for this connection
     * @return Vector      the result of the operation
     */
    private Vector removeUser(String username, String identifierId) {
        boolean result;

        try {
            result = getAdminConnection(identifierId).removeUser(username);
            return pack(Boolean.TRUE, new Boolean(result));
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Return a list of all registered users.
     *
     * @param identifierId  the unique identifier for this connection
     * @return Vector         The result of the operation
     */
    private Vector getAllUsers(String identifierId) {
        Vector result;

        try {
            result = getAdminConnection(identifierId).getAllUsers();
            return pack(Boolean.TRUE, result);
        } catch (JMSException exception) {
            return pack(Boolean.FALSE, exception);
        }
    }

    /**
     * Pack all the data that is required by the server in a vector.
     * Set the size of the vector to be exactly the right size for efficiency.
     *
     * @param success Boolean indicating success or failure of request.
     * @param ob The Object being returned.
     * @return Vector The vector containing all the data.
     *
     */
    private Vector pack(Boolean success, Object ob) {
        Vector v = new Vector(2);
        v.add(success);
        v.add(ob);
        return v;
    }

    /**
     * Get the AdminConnection for this connection.
     *
     * @param identifierId the unique id for this connection
     * @return AdminConnection the connection
     * @throws JMSSecurityException if the connection not exist
     */
    private AdminConnection getAdminConnection(String identifierId)
        throws JMSSecurityException {

        AdminConnection connection =
            AdminConnectionManager.instance().getConnection(identifierId);

        if (connection == null) {
            throw new JMSSecurityException("Not authorized for this operation");
        }

        return connection;
    }
}
