/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: IpcJndiServer.java,v 1.3 2003/08/07 13:33:09 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */

package org.exolab.jms.server.mipc;

import java.io.IOException;

import javax.naming.Context;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.exolab.core.ipc.Server;
import org.exolab.jms.config.Configuration;
import org.exolab.jms.config.ConfigurationManager;
import org.exolab.jms.config.TcpConfigurationType;
import org.exolab.jms.jndi.JndiServerIfc;


/**
 * This class enables clients to interface to the embedded JNDI provider
 * using the TCP connector
 *
 * @version     $Revision: 1.3 $ $Date: 2003/08/07 13:33:09 $
 * @author      <a href="mailto:mourikis@exolab.org">Jim Mourikis</a>
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 */
public class IpcJndiServer implements JndiServerIfc {

    /**
     * The connection server
     */
    private Server _server;

    /**
     * The root context
     */
    private Context _context;

    /**
     * The logger
     */
    private static final Log _log = LogFactory.getLog(IpcJndiServer.class);


    /**
     * Construct a new <code>IpcJndiServer</code>
     *
     * @param context the root context
     */
    public IpcJndiServer(Context context) {
        _context = context;
    }

    /**
     * This method is called to initialise the JNDI service.
     *
     * @throws NamingException if the service cannot be initialised
     */
    public void init() throws NamingException {
        TcpConfigurationType tcp = getTcpConfiguration();
        int port = tcp.getJndiPort();

        try {
            _server = new Server(port, new NameServiceProvider(_context));
        } catch (IOException exception) {
            NamingException error = new NamingException(
                "Failed to initialise TCP JNDI interface");
            error.setRootCause(exception);
            throw error;
        }

        // Start the jndi service
        Thread t = new Thread(_server);
        t.start();
        _log.info("Started TCP JNDI server on port " + port);
    }

    /**
     * Returns the tcp configuration
     *
     * @return the tcp configuration
     */
    protected TcpConfigurationType getTcpConfiguration() {
        return ConfigurationManager.getConfig().getTcpConfiguration();
    }

} //-- IpcJndiServer
