/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SslIpcJmsServer.java,v 1.6 2003/08/07 13:33:09 tanderson Exp $
 *
 * Date         Author  Changes
 * $Date	    jimm    Created
 */

package org.exolab.jms.server.mipc;

import java.io.IOException;

import org.exolab.core.mipc.MultiplexConnectionServerIfc;
import org.exolab.core.mipc.MultiplexSSLConnectionServer;
import org.exolab.jms.config.ConfigurationManager;
import org.exolab.jms.config.TcpConfigurationType;
import org.exolab.jms.config.types.SchemeType;


/**
 * Open an MIPC connection server using SSL.
 *
 * @version     $Revision: 1.6 $ $Date: 2003/08/07 13:33:09 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 * @see         org.exolab.jms.client.mipc.SslJmsServerStub
 * @see         org.exolab.jms.server.mipc.IpcJmsServer
 */
public class SslIpcJmsServer extends IpcJmsServer {

    /**
     * Construct a new <code>SslIpcJmsServer</code>
     */
    public SslIpcJmsServer() {
    }

    /**
     * Create an SSL connection.
     *
     * @param port The port number to use.
     * @return <code>MultiplexConnectionServerIfc</code> The created
     * connection.
     * @exception IOException If the server fails to initialise the
     * ip service.
     */
    protected MultiplexConnectionServerIfc createServer(int port)
        throws IOException {
        return new MultiplexSSLConnectionServer(port, this);
    }

    /**
     * Returns the connector scheme for this server.
     */
    protected SchemeType getScheme() {
        return SchemeType.TCPS;
    }

    /**
     * Returns the TCP configuration
     *
     * @return the TCP configuration
     */
    protected TcpConfigurationType getTcpConfiguration() {
        return ConfigurationManager.getConfig().getTcpsConfiguration();
    }

} //-- SslIpcJmsServer
