/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RemoteJmsAdminServerIfc.java,v 1.11 2003/08/07 13:33:10 tanderson Exp $
 *
 * Date         Author  Changes
 * 05/10/2000   jima    Created
 */
package org.exolab.jms.server.rmi;

import java.rmi.Remote;
import java.rmi.RemoteException;

import javax.jms.JMSException;


/**
 * This specifies all the administration methods that can be used to control
 * the JMS server through an RMI connector. The control logic is all at the
 * org.exolab.jms.server package level
 *
 * @version     $Revision: 1.11 $ $Date: 2003/08/07 13:33:10 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         java.rmi.Remote
 **/
public interface RemoteJmsAdminServerIfc
    extends Remote {

    /**
     * Create a connection to the specified server. This will create an
     * instance of a JmsServerConnection and then return a remote reference
     * to it.
     *
     * @param id  client identity
     * @param username the user's name
     * @param password the user's password
     * @return RemoteJmsAdminConnectionIfc
     * @throws JMSException if the authentication failed
     * @throws RemoteException if failed to export object
     */
    RemoteJmsAdminConnectionIfc createConnection(
        String id, String username, String password)
        throws JMSException, RemoteException;

}



