/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RemoteJmsServerConnectionIfc.java,v 1.7 2003/08/17 01:32:26 tanderson Exp $
 *
 * Date         Author  Changes
 * 04/18/2000   jima    Created
 */
package org.exolab.jms.server.rmi;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Enumeration;

import javax.jms.JMSException;


/**
 * This class RMI enables the JmsServerConnection by allowing clients to access
 * the object across another VM. It basically has been grafted off the
 * JmsServerConnection class with the RemoteException appended to the message
 * signature.
 *
 * @version     $Revision: 1.7 $ $Date: 2003/08/17 01:32:26 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jms.server.JmsServerConnection
 */
public interface RemoteJmsServerConnectionIfc
    extends Remote {

    /**
     * Create and return a sesion for this connection  Assign the connection's
     * clientId to the sessions.
     * <p>
     * The created session represents an endpoint with the JmsServer where
     * you can create consumers, producers and destinations. If there is an
     * error creating the session then RemoteException is thrown
     *
     * @param       ackMode         the ack mode for the session
     * @return      JmsSession      created session
     * @exception   JMSException
     * @exception   RemoteException
     */
    public RemoteJmsServerSessionIfc createSession(int ackMode, boolean transacted)
        throws JMSException, RemoteException;

    /**
     * Delete the specified session from the server. This will destroy all
     * associated consumers and producers.
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @param       session         session to delete
     * @exception   JMSException
     * @exception   RemoteException
     */
    public void deleteSession(RemoteJmsServerSessionIfc session)
        throws JMSException, RemoteException;

    /**
     * Return the number of session objects currently allocated to this
     * connection.
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @return      int
     * @exception   JMSException
     * @exception   RemoteException
     */
    public int getSessionCount()
        throws JMSException, RemoteException;

    /**
     * Return an enumeration of RemoteJmsServerSessionIfc objects currently
     * allocated to this session.
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @return      Enumeration
     * @exception   JMSException
     * @exception   RemoteException
     */
    public Enumeration getSessions()
        throws JMSException, RemoteException;

    /**
     * Start all the sessions allocated to this connection.
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @exception   JMSException
     * @exception   RemoteException
     */
    public void start()
        throws JMSException, RemoteException;

    /**
     * Stop all the sessions allocated to this collection. Do not reemove
     * the session
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @exception   JMSException
     * @exception   RemoteException
     */
    public void stop()
        throws JMSException, RemoteException;

    /**
     * Close all the sessions allocated to this collection. This will also
     * remove the session from the allocated list of sessions
     * <p>
     * If there is a problem completing this request then throw RemoteException
     * exception
     *
     * @exception   JMSException
     * @exception   RemoteException
     */
    public void close()
        throws JMSException, RemoteException;

    /**
     * Retrieve the identity of this conection.
     *
     * @return      String
     * @exception   RemoteException
     */
    public String getConnectionId()
        throws RemoteException;

    /**
     * Send a ping to the server.
     *
     * @exception   RemoteException
     */
    public void ping()
        throws RemoteException;
}



