/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RmiJndiServer.java,v 1.4 2003/08/07 13:33:10 tanderson Exp $
 *
 * Date         Author  Changes
 * 3/14/2000    jima    Created
 */
package org.exolab.jms.server.rmi;

import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.server.UnicastRemoteObject;

import javax.naming.Context;
import javax.naming.NamingException;

import org.apache.avalon.excalibur.naming.rmi.RMINamingProvider;
import org.apache.avalon.excalibur.naming.rmi.server.RMINamingProviderImpl;

import org.exolab.jms.config.ConfigHelper;
import org.exolab.jms.config.Configuration;
import org.exolab.jms.config.ConfigurationManager;
import org.exolab.jms.config.types.SchemeType;
import org.exolab.jms.jndi.JndiServerIfc;


/**
 * This class enables clients to interface to the embedded JNDI provider
 * using the RMI connector
 *
 * @version     $Revision: 1.4 $ $Date: 2003/08/07 13:33:10 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 */
public class RmiJndiServer
    implements JndiServerIfc {

    /**
     * The root context
     */
    private Context _context;

    /**
     * The provider implementation
     */
    private RMINamingProvider _provider;


    /**
     * Construct a new <code>RmiJndiServer</code>
     *
     * @param context the root context
     */
    public RmiJndiServer(Context context) {
        _context = context;
    }

    /**
     * This method is called to initialise the JNDI service.
     *
     * @throws NamingException if the service cannot be initialised
     */
    public void init() throws NamingException {
        Configuration config = ConfigurationManager.getConfig();
        _provider = new RMINamingProviderImpl(_context);

        try {
            // export and bind the provider to the registry
            Remote proxy = UnicastRemoteObject.exportObject(_provider);
            String url = ConfigHelper.getJndiURL(SchemeType.RMI, config);
            Naming.rebind(url, proxy);
        } catch (Exception exception) {
            NamingException error = new NamingException(
                "Failed to bind the RMI JNDI interface in the registry");
            error.setRootCause(exception);
            throw error;
        }
    }

}
