/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: NullTransactionManager.java,v 1.2 2003/08/07 13:33:12 tanderson Exp $
 *
 * Date         Author  Changes
 * 07/02/2001   jima    Created
 */
package org.exolab.jms.transaction;

import javax.transaction.Synchronization;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.xa.XAResource;


/**
 * This class instantiates the NullTransactionManager, which is used in place
 * of a real transaction manager. In particular it is used for the JDBM database
 * <p>
 * This is really an interim solution to support JDBM, with most of the methods
 * having an empty body.
 *
 * @version     $Revision: 1.2 $ $Date: 2003/08/07 13:33:12 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public class NullTransactionManager implements TransactionManager {

    // implementation of TransactionManager.begin
    public void begin() {
        // null operation
    }

    // implementation of TransactionManager.commit
    public void commit() {
        // null operation
    }

    // implementation of TransactionManager.rollback
    public void rollback() {
        // null operation
    }

    // implementation of TransactionManager.getStatus
    public int getStatus() {
        return 0;
    }

    // implementation of TransactionManager.getTransaction
    public Transaction getTransaction() {
        return new NullTransaction();
    }

    // implementation of TransactionManager.resume
    public void resume(Transaction transaction) {
        // null operation
    }

    // implementation of TransactionManager.setRollbackOnly
    public void setRollbackOnly() {
        // null operation
    }

    // implementation of TransactionManager.setTransactionTimeout
    public void setTransactionTimeout(int timeout) {
        // null implementation
    }

    // implementation of TransactionManager.suspend
    public Transaction suspend() {
        return new NullTransaction();
    }
}


/**
 * The NullTransaction is part of the NullTransactionManager support
 * with most methods implemented with empty bodies
 */
class NullTransaction implements javax.transaction.Transaction {

    // implementation Transaction.commit
    public void commit() {
        // null implementation
    }

    // implementation Transaction.delistResource
    public boolean delistResource(XAResource a, int b) {
        return false;
    }

    // implementation Transaction.rollback
    public void rollback() {
        // null implementation
    }

    // implementation Transaction.enlistResource
    public boolean enlistResource(XAResource a) {
        return false;
    }

    // implementation Transaction.getStatus
    public int getStatus() {
        return 0;
    }

    // implementation Transaction.registerSynchronization
    public void registerSynchronization(Synchronization a) {
        // null implementation
    }

    // implementation Transaction.setRollbackOnly
    public void setRollbackOnly() {
        // null implementation
    }
}
