/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 *
 * $Id: TestExternalXid.java,v 1.1 2003/02/06 13:42:47 tanderson Exp $
 *
 * Date			Author  Changes
 * 2003-02-04   Jem     Initial Version
 */
package org.exolab.jms.tranlog;

import junit.framework.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Externalizable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.transaction.xa.Xid;



/**
 *
 * Basic Unit Test - needs loads more functionality :-)
 */

public class TestExternalXid extends TestCase
{

    private static final  int   format = 8;
    private static final  byte[] global = {'1','2','3','4','5','6'};
    private static final  byte[] branch = {'a','b','c' };

  public TestExternalXid(String s)
  {
    super(s);
  }

  protected void setUp()
  {
  }

  protected void tearDown()
  {
  }

  public void testEquals()
  {

      ExternalXid x1 = new ExternalXid(format, global, branch);
      ExternalXid x2;

      // Test Equals self

      assertTrue("Failed self-equality", x1.equals(x1));

      // Test content equality

      x2 = new ExternalXid(format, global, branch);
      assertTrue("Failed Content Equality", x1.equals(x2));

      // Copy Construct

      x2 = new ExternalXid(x1);
      assertTrue("Failed Copy Construct Equality", x1.equals(x2));
  }

  public void testExternalization() {

      try {

      ObjectInputStream     oIn;
      ObjectOutputStream    oOut;
      ByteArrayOutputStream baos = null;
      ByteArrayInputStream  bais = null;

      // Build an instance
      Externalizable x1 = new ExternalXid(format, global, branch);

      // Externalize it
      baos = new ByteArrayOutputStream();
      oOut = new ObjectOutputStream(baos);

      x1.writeExternal(oOut);
      oOut.close();

      // Read it back
      bais = new ByteArrayInputStream(baos.toByteArray());
      oIn  = new ObjectInputStream(bais);

      Externalizable x2 = new ExternalXid();
      x2.readExternal(oIn);

      // Equality check
      assertEquals("Not equal() after externalization", x1, x2);

      }
      catch (Throwable t) {
          fail(t.getMessage());
      }
  }

  /**
   * Test for equality of Strings and hashCodes
   *
   */
  public void testOtherEquality() {

      Xid x1 = new ExternalXid(format, global, branch);
      Xid x2 = new ExternalXid(format, global, branch);

      int hash1, hash2;

      hash1 = x1.hashCode();
      hash2 = x2.hashCode();

      assertEquals("Hashcode mismatch", hash1, hash2);

      String s1, s2;

      s1 = x1.toString();
      s2 = x2.toString();

      assertEquals("toString() mismatch", s1, s2);
  }
}
