/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestBug665495.java,v 1.4 2003/08/16 09:44:52 tanderson Exp $
 *
 * Date         Author      Changes
 * 21/02/2002   jima        Created
 */

package org.exolab.testharness.jms.bugs;

// junit library
import junit.framework.TestCase;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.InitialContext;

// jms library
import javax.jms.DeliveryMode;
import javax.jms.Session;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.Topic;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.QueueRequestor;
import javax.jms.QueueSender;
import javax.jms.QueueReceiver;
import javax.jms.TextMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.JMSException;
import javax.jms.IllegalStateException;

// openjms
import org.exolab.jms.administration.JmsAdminServerIfc;

// openjms testhaness
import org.exolab.testharness.jms.util.ClientHelper;
import org.exolab.testharness.jms.util.CountingListener;


/**
 * This tests for http://sourceforge.net/tracker/index.php?func=detail&aid=665495&group_id=54559&atid=474136
 *
 * @version     $Revision: 1.4 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 */
public class TestBug665495  extends TestCase {

    /**
     * Constructor
     */
    public TestBug665495(String name) {
        super(name);
    }

    /** 
     * Test the bug for queues
     */
    public void testQueue() 
        throws Exception {
        final String name = "testQueue665495";
        final String message = "Hello testQueue665495";
        final QueueConnection connection = ClientHelper.getQueueConnection();
        try {
            connection.start();
            
            Thread sender = new Thread(new Runnable() {
                // implementation of Runnable.run
                public void run() {
                    try {
                        QueueSession session = connection.createQueueSession(
                            false, Session.CLIENT_ACKNOWLEDGE);
    
                        // create a queue sender and send a message
                        QueueSender sender = session.createSender(null);
                        sender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
                        sender.send(session.createQueue(name),
                                    session.createTextMessage(message));
                    } catch (Exception exception) {
                        exception.printStackTrace();
                        assertTrue("TestBug665495.testQueue : " + 
                                   exception.getMessage(), false);
                    }
                }
            });

            Thread receiver = new Thread(new Runnable() {
                // implementation of Runnable.run
                public void run() {          
                    try {
                        QueueSession session = connection.createQueueSession(
                            false, Session.CLIENT_ACKNOWLEDGE);
    
                        // create a queue sender and send a message
                        QueueReceiver receiver = session.createReceiver(
                            session.createQueue(name));
                        assertNotNull(receiver.receive());
                    } catch (Exception exception) {
                        exception.printStackTrace();
                        assertTrue("TestBug665495.testQueue : " + 
                                   exception.getMessage(), false);
                    }
                }
            });

            // start the receiver and then after 3 seconds start the sender
            receiver.start();
            Thread.currentThread().sleep(3000);
            sender.start();

            // wait for the two to complete
            receiver.join();
            sender.join();
        } catch (Exception exception) {
            exception.printStackTrace();
            assertTrue("TestBug665495.testQueue : " + exception.getMessage(), 
                       false);
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (JMSException ignore) {
                }
            }
        }
    }

    /** 
     * Test the bug for topics
     */
    public void testTopic() 
    throws Exception {
        final String name = "testTopic665495";
        final String message = "Hello testTopic665495";
        final TopicConnection connection = ClientHelper.getTopicConnection();
        try {
            connection.start();

            Thread publisher = new Thread(new Runnable() {
                // implementation of Runnable.run
                public void run() {
                    try {
                        TopicSession session = connection.createTopicSession(
                            false, Session.CLIENT_ACKNOWLEDGE);
                        Topic topic = session.createTopic(name);
                        TopicPublisher publisher = session.createPublisher(
                            null);
                        publisher.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
                        publisher.publish(topic, session.createTextMessage(
                            message));
                    } catch (Exception exception) {
                            exception.printStackTrace();
                            assertTrue("TestBug665495.testTopic : " + 
                                       exception.getMessage(), false);
                        }
                    }
                    });

            Thread consumer = new Thread(new Runnable() {
                    // implementation of Runnable.run
                    public void run() {          
                        try {
                            TopicSession session = connection.createTopicSession(
                                false, Session.CLIENT_ACKNOWLEDGE);
                            Topic topic = session.createTopic(name);
                            TopicSubscriber subscriber = session.createSubscriber(topic);
                            assertNotNull(subscriber.receive());
                        } catch (Exception exception) {
                            exception.printStackTrace();
                            assertTrue("TestBug665495.testTopic : " + 
                                       exception.getMessage(), false);
                        }
                    }
                    });

            // start the receiver and then after 3 seconds start the sender
            consumer.start();
            Thread.currentThread().sleep(3000);
            publisher.start();

            // wait for the two to complete
            consumer.join();
            publisher.join();
        } catch (Exception exception) {
            exception.printStackTrace();
            assertTrue("TestBug665495.testTopic : " + exception.getMessage(), 
                       false);
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (JMSException ignore) {
                }
            }
        }
    }
}

