/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestBug719.java,v 1.1 2001/09/18 10:55:08 jima Exp $
 *
 * Date         Author      Changes
 * 09/04/2001   jima        Created
 */

package org.exolab.testharness.jms.bugs;

// junit library
import junit.framework.TestCase;

// jms library
import javax.jms.Session;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.Topic;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.QueueRequestor;
import javax.jms.QueueSender;
import javax.jms.TextMessage;
import javax.jms.StreamMessage;
import javax.jms.BytesMessage;
import javax.jms.MapMessage;
import javax.jms.ObjectMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.JMSException;
import javax.jms.IllegalStateException;

// openjms testhaness
import org.exolab.testharness.jms.util.ClientHelper;

/**
 * This tests bug #719 http://bugzilla.exolab.org/show_bug.cgi?id=719
 *
 * @version     $Revision: 1.1 $ $Date: 2001/09/18 10:55:08 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jtf.CWTestCateogry
 * @see         org.exolab.jtf.CWTestCase
 */
public class TestBug719 extends TestCase {

    /**
     * Constructor
     */
    public TestBug719(String name) {
        super(name);
    }

    /** 
     * Test the bug for queues
     */
    public void testQueue() {
        try {
            QueueConnection connection = ClientHelper.getQueueConnection();
            connection.start();

            // execute some methods on the connection
            connection.createQueueSession(true, Session.CLIENT_ACKNOWLEDGE);
            connection.getMetaData();
            connection.stop();

            // close the connection
            connection.close();
            
            // try using the connection after it's closed
            try {
                try {
                    connection.createQueueSession(true, Session.CLIENT_ACKNOWLEDGE);
                    assertTrue("TestBug719.testQueue : calling createQueueSession on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.getMetaData();
                    assertTrue("TestBug719.testQueue : calling getMetaData on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.start();
                    assertTrue("TestBug719.testQueue : calling start on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.close();
                } catch (IllegalStateException ignore) {
                    assertTrue("TestBug719.testQueue : calling close on closed connection", false);
                }
            } catch (Exception exception) {
                assertTrue("TestBug719.testQueue : " + exception.getMessage(), false);
            }
        } catch (Exception exception) {
            assertTrue("TestBug719.testQueue : " + exception.getMessage(), false);
        }
    }

    /** 
     * Test the bug for topics
     */
    public void testTopic() {
        try {
            TopicConnection connection = ClientHelper.getTopicConnection();
            connection.start();

            // execute some methods on the connection
            connection.createTopicSession(true, Session.CLIENT_ACKNOWLEDGE);
            connection.getClientID();
            connection.getMetaData();
            connection.stop();

            // close the connection
            connection.close();
            
            // try using the connection after it's closed
            try {
                try {
                    connection.createTopicSession(true, Session.CLIENT_ACKNOWLEDGE);
                    assertTrue("TestBug719.testTopic : calling createQueueSession on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.getMetaData();
                    assertTrue("TestBug719.testTopic : calling getMetaData on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.start();
                    assertTrue("TestBug719.testTopic : calling start on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.getClientID();
                    assertTrue("TestBug719.testTopic : calling start on closed connection", false);
                } catch (IllegalStateException ignore) {
                }
            
                try {
                    connection.close();
                } catch (IllegalStateException ignore) {
                    assertTrue("TestBug719.testTopic : calling close on closed connection", false);
                }
            } catch (Exception exception) {
                assertTrue("TestBug719.testTopic : " + exception.getMessage(), false);
            }
        } catch (Exception exception) {
            assertTrue("TestBug719.testTopic : " + exception.getMessage(), false);
        }
    }
}


