/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestBug888.java,v 1.2 2003/01/16 13:39:08 tanderson Exp $
 *
 * Date         Author      Changes
 * 21/02/2002   jima        Created
 */

package org.exolab.testharness.jms.bugs;

// junit library
import junit.framework.TestCase;

// jms library
import javax.jms.DeliveryMode;
import javax.jms.Session;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.Topic;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.QueueRequestor;
import javax.jms.QueueSender;
import javax.jms.TextMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.JMSException;
import javax.jms.IllegalStateException;

// openjms testhaness
import org.exolab.testharness.jms.util.ClientHelper;

/**
 * This tests bug #888 http://bugzilla.exolab.org/show_bug.cgi?id=888
 *
 * @version     $Revision: 1.2 $ $Date: 2003/01/16 13:39:08 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jtf.CWTestCateogry
 * @see         org.exolab.jtf.CWTestCase
 */
public class TestBug888  extends TestCase {

    /**
     * Constructor
     */
    public TestBug888(String name) {
        super(name);
    }

    /** 
     * Test the bug for queues
     */
    public void testQueue() {
        try {
            QueueConnection connection = ClientHelper.getQueueConnection();
            connection.start();

            QueueSession sendSession = connection.createQueueSession(
                true, Session.CLIENT_ACKNOWLEDGE);
            QueueSession session = connection.createQueueSession(
                true, Session.CLIENT_ACKNOWLEDGE);
            Queue queue = session.createQueue("QueueBug888");

            // create the receiver and set a message listener
            QueueReceiver receiver = session.createReceiver(queue);
            MyMessageListener listener = new MyMessageListener(session);
            receiver.setMessageListener(listener);

            // create a queue sender and publish a message
            QueueSender sender = sendSession.createSender(queue);
            sender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
            for (int index = 0; index < 100; index++) {
                TextMessage message = sendSession.createTextMessage(
                    "Testing Bug888 " + index);
                sender.send(message);
            }
            sendSession.commit();

            // loop for a maximum of 100 seconds while we wait for 
            // the receiver to receive 150 messages
            for (int index = 0; index < 1000; index ++) {
                if (listener.getMessageCount() == 150) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }

            assertEquals(
                "Listener didn't receive expected no. of messages",
                150, listener.getMessageCount());

            connection.close();
        } catch (Exception exception) {
            exception.printStackTrace();
            assertTrue("TestBug888.testQueue : " + exception.getMessage(), false);
        }
    }

    /** 
     * Test the bug for topics
     */
    public void testTopic() {
        try {
            TopicConnection connection = ClientHelper.getTopicConnection();
            connection.start();

            TopicSession sendSession = connection.createTopicSession(true, 
                                        Session.CLIENT_ACKNOWLEDGE);
            TopicSession session = connection.createTopicSession(true, 
                                        Session.CLIENT_ACKNOWLEDGE);
            Topic topic = session.createTopic("TopicBug888");
            
            // create the consumer
            TopicSubscriber subscriber = session.createSubscriber(topic);
            MyMessageListener listener = new MyMessageListener(session);
            subscriber.setMessageListener(listener);

            // create the publisher
            TopicPublisher publisher = sendSession.createPublisher(topic);
            publisher.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
            for (int index = 0; index < 100; index++) {
                TextMessage message = sendSession.createTextMessage("Testing Bug888 " + index);
                publisher.publish(message);
            }
            sendSession.commit();

            // loop for a maximum of 100 seconds while we wait for 
            // the consumer to receive 150 messages
            for (int index = 0; index < 1000; index ++) {
                if (listener.getMessageCount() == 150) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }

            assertEquals(
                "Listener didn't receive expected no. of messages",
                150, listener.getMessageCount());
            connection.close();
        } catch (Exception exception) {
            assertTrue("TestBug888.testTopic : " + exception.getMessage(), false);
        }
    }
}


/**
 * A private listener class
 */
class MyMessageListener implements MessageListener {
    /**
     * Maintains the number of messages received by the listener
     */
    private int _messagesReceived;

    /**
     * Caches the session
     */
    private Session _session = null;

    
    /**
     * Construct a listener for the specified session
     */
    MyMessageListener(Session session) {
        _session = session;
    }
    

    /**
     * Return the number of received messages
     *
     * @return int
     */
    int getMessageCount() {
        return _messagesReceived;
    }

    /** Pass a message to the Listener.
      *
      * @param message the message passed to the listener.
      */
    public void onMessage(Message message) {
        try {
            ++_messagesReceived;
            if (_messagesReceived == 50) {
                // commit after received 50 messages
                _session.commit();
            } else if (_messagesReceived == 100) {
                // rollback after the 100th
                _session.rollback();
            }
        } catch (Exception exception) {
            exception.printStackTrace();
        }
        
    }
}


