/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestBug889.java,v 1.6 2003/03/24 12:16:10 tanderson Exp $
 *
 * Date         Author      Changes
 * 21/02/2002   jima        Created
 */

package org.exolab.testharness.jms.bugs;

import javax.naming.Context;
import javax.naming.NamingException;

import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

import junit.framework.TestCase;

import org.exolab.testharness.jms.util.ClientHelper;
import org.exolab.testharness.jms.util.CountingListener;


/**
 * This tests bug #889 http://bugzilla.exolab.org/show_bug.cgi?id=889
 *
 * @version     $Revision: 1.6 $ $Date: 2003/03/24 12:16:10 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jtf.CWTestCateogry
 * @see         org.exolab.jtf.CWTestCase
 */
public class TestBug889  extends TestCase {

    /**
     * Constructor
     */
    public TestBug889(String name) {
        super(name);
    }

    /** 
     * Test the bug for queues
     */
    public void testQueue() throws NamingException, JMSException {
        final String name = "testQueue889";
        QueueConnection connection = ClientHelper.getQueueConnection();
        connection.start();

        for (int i = 0; i < 5; ++i) {
            ClientHelper.setupDestination(name, Boolean.TRUE);

            QueueSession session = connection.createQueueSession(
                true, Session.CLIENT_ACKNOWLEDGE);
            

            Context context = ClientHelper.getInitialContext();
            Queue queue = (Queue) context.lookup(name);

            // create the receiver and set a message listener
            QueueReceiver receiver = session.createReceiver(queue);
            CountingListener listener = new CountingListener();

            // create a queue sender and publish a message
            QueueSender sender = session.createSender(queue);
            for (int index = 0; index < 10; index++) {
                TextMessage message = session.createTextMessage(
                    "Testing testQueue889 " + index);
                sender.send(message);
            }
            session.commit();
                
            receiver.setMessageListener(listener);

            // loop for a maximum of 10 seconds while we wait for 
            // the receiver to receive 10 messages
            for (int index = 0; index < 100; index ++) {
                if (listener.getMessageCount() == 10) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }

            assertEquals(
                "Listener didn't receive expected no. of messages",
                10, listener.getMessageCount());

            session.rollback();
            
            // loop for a maximum of 10 seconds while we wait for 
            // the receiver to receive the 10 messages again
            for (int index = 0; index < 100; index ++) {
                if (listener.getMessageCount() == 20) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }

            assertEquals(
                "Listener didn't receive expected no. of messages",
                20, listener.getMessageCount());
            session.close();
        }
        connection.close();
    }

    /** 
     * Test the bug for queues
     */
    public void testTopic() throws NamingException, JMSException {
        final String name = "testTopic889";
        TopicConnection connection = ClientHelper.getTopicConnection();
        connection.start();

        for (int i = 0; i < 5; ++i) {
            ClientHelper.setupDestination(name, Boolean.FALSE);

            TopicSession session = connection.createTopicSession(
                true, Session.CLIENT_ACKNOWLEDGE);
            
            Context context = ClientHelper.getInitialContext();
            Topic topic = (Topic)context.lookup(name);

            // create the subscriber and message listener
            TopicSubscriber subscriber = session.createSubscriber(topic);
            CountingListener listener = new CountingListener();

            // create a publisher and send the messages
            TopicPublisher publisher = session.createPublisher(topic);
            for (int index = 0; index < 10; index++) {
                TextMessage message = session.createTextMessage(
                    "Testing testTopic889 " + index);
                publisher.publish(message);
            }
            session.commit();
            
            subscriber.setMessageListener(listener);
            
            // loop for a maximum of 10 seconds while we wait for 
            // the receiver to receive the 10 messages
            for (int index = 0; index < 100; index ++) {
                if (listener.getMessageCount() == 10) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }
            
            assertEquals(
                "Listener didn't receive expected no. of messages",
                10, listener.getMessageCount());
            
            session.rollback();
            
            // loop for a maximum of 10 seconds while we wait for 
            // the receiver to receive the 10 messages again
            for (int index = 0; index < 100; index ++) {
                if (listener.getMessageCount() == 20) {
                    break;
                } else {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ignore) {
                    }
                }
            }
            
            assertEquals(
                "Listener didn't receive expected no. of messages",
                20, listener.getMessageCount());
            
            session.close();
        }
        connection.close();
    }
    
}
