/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestBug894.java,v 1.4 2003/03/24 12:16:10 tanderson Exp $
 *
 * Date         Author      Changes
 * 21/02/2002   jima        Created
 */

package org.exolab.testharness.jms.bugs;

// junit library
import junit.framework.TestCase;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.InitialContext;

// jms library
import javax.jms.DeliveryMode;
import javax.jms.Session;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.Topic;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.QueueRequestor;
import javax.jms.QueueSender;
import javax.jms.TextMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.JMSException;
import javax.jms.IllegalStateException;

import org.exolab.testharness.jms.util.ClientHelper;
import org.exolab.testharness.jms.util.CountingListener;


/**
 * This tests bug #888 http://bugzilla.exolab.org/show_bug.cgi?id=888
 *
 * @version     $Revision: 1.4 $ $Date: 2003/03/24 12:16:10 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jtf.CWTestCateogry
 * @see         org.exolab.jtf.CWTestCase
 */
public class TestBug894  extends TestCase {

    /**
     * Constructor
     */
    public TestBug894(String name) {
        super(name);
    }

    /** 
     * Test the bug for queues
     */
    public void testQueue() {
        final String name = "testQueue894";
        try {
            QueueConnection connection = ClientHelper.getQueueConnection();
            connection.start();

            for (int i = 0; i < 6; ++i) {
                Queue queue = null;
                boolean persistent = false;
                if (i % 2 == 0) {
                    ClientHelper.setupDestination(name, Boolean.TRUE);
                    Context context = ClientHelper.getInitialContext();
                    queue = (Queue) context.lookup(name);
                    persistent = true;
                } else {
                    ClientHelper.removeDestination(name);
                }

                QueueSession session = connection.createQueueSession(
                    true, Session.CLIENT_ACKNOWLEDGE);
                if (queue == null) {
                    queue = session.createQueue(name);
                }

                // create the receiver and set a message listener
                QueueReceiver receiver = session.createReceiver(queue);
                CountingListener listener = new CountingListener();
                receiver.setMessageListener(listener);

                Thread.currentThread().sleep(1000);

                assertTrue(listener.getMessageCount() == 0);

                // create a queue sender and publish some messages
                QueueSender sender = session.createSender(queue);
                if (!persistent) {
                    sender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
                }
                send(session, sender, 10);

                // rollback the session, and verify the listener doesn't
                // receive any messages
                session.rollback();            
                Thread.currentThread().sleep(2000);
                assertTrue(listener.getMessageCount() == 0);

                // send 10 messages, and commit. The listener should
                // receive 10 messages
                send(session, sender, 10);
                session.commit();

                Thread.currentThread().sleep(2000);

                assertTrue(listener.getMessageCount() == 10);
                session.rollback();
                Thread.currentThread().sleep(5000);
                session.close();
            }
            connection.close();
        } catch (Exception exception) {
            exception.printStackTrace();
            assertTrue("TestBug894.testQueue : " + exception.getMessage(), 
                       false);
        }
    }

    /** 
     * Test the bug for topics
     */
    public void testTopic() {
        final String name = "testTopic894";
        try {
            TopicConnection connection = ClientHelper.getTopicConnection();
            connection.start();

            for (int i = 0; i < 6; ++i) {
                Topic topic = null;
                boolean persistent = false;
                if (i % 2 == 0) {
                    ClientHelper.setupDestination(name, Boolean.FALSE);
                    Context context = ClientHelper.getInitialContext();
                    topic = (Topic) context.lookup(name);
                    persistent = true;
                } else {
                    ClientHelper.removeDestination(name);
                }

                TopicSession session = connection.createTopicSession(
                    true, Session.CLIENT_ACKNOWLEDGE);
                if (topic == null) {
                    topic = session.createTopic(name);
                }

                // create the receiver and set a message listener
                TopicSubscriber subscriber = session.createSubscriber(topic);
                CountingListener listener = new CountingListener();
                subscriber.setMessageListener(listener);

                Thread.currentThread().sleep(1000);

                assertTrue(listener.getMessageCount() == 0);

                // create a queue sender and publish some messages
                TopicPublisher publisher = session.createPublisher(topic);
                if (!persistent) {
                    publisher.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
                }
                publish(session, publisher, 10);

                // rollback the session, and verify the listener doesn't
                // receive any messages
                session.rollback();            
                Thread.currentThread().sleep(2000);
                assertTrue(listener.getMessageCount() == 0);

                // send 10 messages, and commit. The listener should
                // receive 10 messages
                publish(session, publisher, 10);
                session.commit();

                Thread.currentThread().sleep(2000);

                assertTrue(listener.getMessageCount() == 10);
                session.rollback();
                Thread.currentThread().sleep(5000);
                session.close();
            }
            connection.close();
        } catch (Exception exception) {
            exception.printStackTrace();
            assertTrue("TestBug894.testTopic : " + exception.getMessage(), 
                       false);
        }
    }

    private void send(QueueSession session, QueueSender sender, int count)
        throws JMSException {
        for (int i = 0; i < 10; ++i) {
            TextMessage message = session.createTextMessage(
                "Test Bug 894 " + i);
            sender.send(message);
        }
    }

    private void publish(TopicSession session, TopicPublisher publisher, int count)
        throws JMSException {
        for (int i = 0; i < 10; ++i) {
            TextMessage message = session.createTextMessage(
                "Test Bug 894 " + i);
            publisher.publish(message);
        }
    }
}

